@@ -5,6 +5,11 @@
  *
  */
 
+#ifndef __STDC_FORMAT_MACROS
+#define __STDC_FORMAT_MACROS
+#endif
+
+#include <stdint.h>
 #include <stdlib.h>
 #include <unistd.h>
 #include <sys/ioctl.h>
@@ -55,10 +60,22 @@ static cString getCA(int value)
     case 0x4AE0:            return cString::sprintf("%s (%X)", "RossCrypt",          value); // Digi Raum Electronics Co. Ltd.
     case 0x5500:            return cString::sprintf("%s (%X)", "Z-Crypt",            value); // Digi Raum Electronics Co. Ltd.
     case 0x5501:            return cString::sprintf("%s (%X)", "Griffin",            value); // Griffin
+    default:                break;
     }
   return cString::sprintf("%X", value);
 }
 
+static const char *getUserString(int Value, const tDvbParameterMap *Map)
+{
+  const tDvbParameterMap *map = Map;
+  while (map && map->userValue != -1) {
+        if (map->driverValue == Value)
+           return map->userString ? trVDR(map->userString) : "---";
+        map++;
+        }
+  return "---";
+}
+
 cString getFrontendInfo(int cardIndex)
 {
   cString info;
@@ -74,15 +91,22 @@ cString getFrontendInfo(int cardIndex)
   int fe = open(dev, O_RDONLY | O_NONBLOCK);
   if (fe < 0)
      return NULL;
-  CHECK(ioctl(fe, FE_GET_INFO, &value));
-  CHECK(ioctl(fe, FE_READ_STATUS, &status));
-  CHECK(ioctl(fe, FE_READ_SIGNAL_STRENGTH, &signal));
-  CHECK(ioctl(fe, FE_READ_SNR, &snr));
-  CHECK(ioctl(fe, FE_READ_BER, &ber));
-  CHECK(ioctl(fe, FE_READ_UNCORRECTED_BLOCKS, &unc));
-  close(fe);
 
-  info = cString::sprintf("CARD:%d\nTYPE:%d\nNAME:%s\nSTAT:%02X\nSGNL:%04X\nSNRA:%04X\nBERA:%08X\nUNCB:%08X", cardIndex, value.type, value.name, status, signal, snr, ber, unc);
+  info = cString::sprintf("CARD:%d", cardIndex);
+
+  if (ioctl(fe, FE_GET_INFO, &value) >= 0)
+     info = cString::sprintf("%s\nTYPE:%d\nNAME:%s", *info, value.type, value.name);
+  if (ioctl(fe, FE_READ_STATUS, &status) >= 0)
+     info = cString::sprintf("%s\nSTAT:%02X", *info, status);
+  if (ioctl(fe, FE_READ_SIGNAL_STRENGTH, &signal) >= 0)
+     info = cString::sprintf("%s\nSGNL:%04X", *info, signal);
+  if (ioctl(fe, FE_READ_SNR, &snr) >= 0)
+     info = cString::sprintf("%s\nSNRA:%04X", *info, snr);
+  if (ioctl(fe, FE_READ_BER, &ber) >= 0)
+     info = cString::sprintf("%s\nBERA:%08X", *info, ber);
+  if (ioctl(fe, FE_READ_UNCORRECTED_BLOCKS, &unc) >= 0)
+     info = cString::sprintf("%s\nUNCB:%08X", *info, unc);
+  close(fe);
 
   if (cFemonOsd::Instance())
      info = cString::sprintf("%s\nVIBR:%.0f\nAUBR:%.0f\nDDBR:%.0f", *info, cFemonOsd::Instance()->GetVideoBitrate(), cFemonOsd::Instance()->GetAudioBitrate(), cFemonOsd::Instance()->GetDolbyBitrate());
@@ -101,7 +125,8 @@ cString getFrontendName(int cardIndex)
   int fe = open(dev, O_RDONLY | O_NONBLOCK);
   if (fe < 0)
      return NULL;
-  CHECK(ioctl(fe, FE_GET_INFO, &value));
+  memset(&value, 0, sizeof(value));
+  ioctl(fe, FE_GET_INFO, &value);
   close(fe);
 
   return (cString::sprintf("%s on device #%d", value.name, cardIndex));
@@ -115,7 +140,8 @@ cString getFrontendStatus(int cardIndex)
   int fe = open(dev, O_RDONLY | O_NONBLOCK);
   if (fe < 0)
      return NULL;
-  CHECK(ioctl(fe, FE_READ_STATUS, &value));
+  memset(&value, 0, sizeof(value));
+  ioctl(fe, FE_READ_STATUS, &value);
   close(fe);
 
   return (cString::sprintf("Status %s:%s:%s:%s:%s on device #%d", (value & FE_HAS_LOCK) ? "LOCKED" : "-", (value & FE_HAS_SIGNAL) ? "SIGNAL" : "-", (value & FE_HAS_CARRIER) ? "CARRIER" : "-", (value & FE_HAS_VITERBI) ? "VITERBI" : "-", (value & FE_HAS_SYNC) ? "SYNC" : "-", cardIndex));
@@ -129,7 +155,7 @@ uint16_t getSignal(int cardIndex)
   int fe = open(dev, O_RDONLY | O_NONBLOCK);
   if (fe < 0)
      return (value);
-  CHECK(ioctl(fe, FE_READ_SIGNAL_STRENGTH, &value));
+  ioctl(fe, FE_READ_SIGNAL_STRENGTH, &value);
   close(fe);
 
   return (value);
@@ -143,7 +169,7 @@ uint16_t getSNR(int cardIndex)
   int fe = open(dev, O_RDONLY | O_NONBLOCK);
   if (fe < 0)
      return (value);
-  CHECK(ioctl(fe, FE_READ_SNR, &value));
+  ioctl(fe, FE_READ_SNR, &value);
   close(fe);
 
   return (value);
@@ -157,7 +183,7 @@ uint32_t getBER(int cardIndex)
   int fe = open(dev, O_RDONLY | O_NONBLOCK);
   if (fe < 0)
      return (value);
-  CHECK(ioctl(fe, FE_READ_BER, &value));
+  ioctl(fe, FE_READ_BER, &value);
   close(fe);
 
   return (value);
@@ -171,7 +197,7 @@ uint32_t getUNC(int cardIndex)
   int fe = open(dev, O_RDONLY | O_NONBLOCK);
   if (fe < 0)
      return (value);
-  CHECK(ioctl(fe, FE_READ_UNCORRECTED_BLOCKS, &value));
+  ioctl(fe, FE_READ_UNCORRECTED_BLOCKS, &value);
   close(fe);
 
   return (value);
@@ -253,6 +279,7 @@ cString getVideoCodec(int value)
   switch (value) {
     case VIDEO_CODEC_MPEG2: return cString::sprintf("%s", tr("MPEG-2"));
     case VIDEO_CODEC_H264:  return cString::sprintf("%s", tr("H.264"));
+    default:                break;
     }
   return cString::sprintf("---");
 }
@@ -267,6 +294,8 @@ cString getAudioCodec(int value)
     case AUDIO_CODEC_MPEG2_II:  return cString::sprintf("%s", tr("MPEG-2 Layer II"));
     case AUDIO_CODEC_MPEG2_III: return cString::sprintf("%s", tr("MPEG-2 Layer III"));
     case AUDIO_CODEC_HEAAC:     return cString::sprintf("%s", tr("HE-AAC"));
+    case AUDIO_CODEC_LATM:      return cString::sprintf("%s", tr("LATM"));
+    default:                    break;
     }
   return cString::sprintf("---");
 }
@@ -278,94 +307,54 @@ cString getAudioChannelMode(int value)
     case AUDIO_CHANNEL_MODE_JOINT_STEREO: return cString::sprintf("%s", tr("joint Stereo"));
     case AUDIO_CHANNEL_MODE_DUAL:         return cString::sprintf("%s", tr("dual"));
     case AUDIO_CHANNEL_MODE_SINGLE:       return cString::sprintf("%s", tr("mono"));
+    default:                              break;
     }
   return cString::sprintf("---");
 }
 
 cString getCoderate(int value)
 {
-  switch (value) {
-    case FEC_NONE: return cString::sprintf("%s", trVDR("none"));
-    case FEC_1_2:  return cString::sprintf("1/2");
-    case FEC_2_3:  return cString::sprintf("2/3");
-    case FEC_3_4:  return cString::sprintf("3/4");
-    case FEC_4_5:  return cString::sprintf("4/5");
-    case FEC_5_6:  return cString::sprintf("5/6");
-    case FEC_6_7:  return cString::sprintf("6/7");
-    case FEC_7_8:  return cString::sprintf("7/8");
-    case FEC_8_9:  return cString::sprintf("8/9");
-    case FEC_AUTO: return cString::sprintf("%s", trVDR("auto"));
-    }
-  return cString::sprintf("---");
+  return cString::sprintf("%s", getUserString(value, CoderateValues));
 }
 
 cString getTransmission(int value)
 {
-  switch (value) {
-    case TRANSMISSION_MODE_2K:   return cString::sprintf("2K");
-    case TRANSMISSION_MODE_8K:   return cString::sprintf("8K");
-    case TRANSMISSION_MODE_AUTO: return cString::sprintf("%s", trVDR("auto"));
-    }
-  return cString::sprintf("---");
+  return cString::sprintf("%s", getUserString(value, TransmissionValues));
 }
- 
+
 cString getBandwidth(int value)
 {
-  switch (value) {
-    case BANDWIDTH_8_MHZ: return cString::sprintf("8 %s", tr("MHz"));
-    case BANDWIDTH_7_MHZ: return cString::sprintf("7 %s", tr("MHz"));
-    case BANDWIDTH_6_MHZ: return cString::sprintf("6 %s", tr("MHz"));
-    case BANDWIDTH_AUTO:  return cString::sprintf("%s", trVDR("auto"));
-    }
-  return cString::sprintf("---");
+  return cString::sprintf("%s", getUserString(value, BandwidthValues));
 }
 
 cString getInversion(int value)
 {
-  switch (value) {
-    case INVERSION_OFF:  return cString::sprintf("%s", tr("off"));
-    case INVERSION_ON:   return cString::sprintf("%s", tr("on"));
-    case INVERSION_AUTO: return cString::sprintf("%s", trVDR("auto"));
-    }
-  return cString::sprintf("---");
+  return cString::sprintf("%s", getUserString(value, InversionValues));
 }
 
 cString getHierarchy(int value)
 {
-  switch (value) {
-    case HIERARCHY_NONE: return cString::sprintf("%s", trVDR("none"));
-    case HIERARCHY_1:    return cString::sprintf("1");
-    case HIERARCHY_2:    return cString::sprintf("2");
-    case HIERARCHY_4:    return cString::sprintf("4");
-    case HIERARCHY_AUTO: return cString::sprintf("%s", trVDR("auto"));
-    }
-  return cString::sprintf("---");
+  return cString::sprintf("%s", getUserString(value, HierarchyValues));
 }
 
 cString getGuard(int value)
 {
-  switch (value) {
-    case GUARD_INTERVAL_1_32: return cString::sprintf("1/32");
-    case GUARD_INTERVAL_1_16: return cString::sprintf("1/16");
-    case GUARD_INTERVAL_1_8:  return cString::sprintf("1/8");
-    case GUARD_INTERVAL_1_4:  return cString::sprintf("1/4");
-    case GUARD_INTERVAL_AUTO: return cString::sprintf("%s", trVDR("auto"));
-    }
-  return cString::sprintf("---");
+  return cString::sprintf("%s", getUserString(value, GuardValues));
 }
 
 cString getModulation(int value)
 {
-  switch (value) {
-    case QPSK:     return cString::sprintf("QPSK");
-    case QAM_16:   return cString::sprintf("QAM 16");
-    case QAM_32:   return cString::sprintf("QAM 32");
-    case QAM_64:   return cString::sprintf("QAM 64");
-    case QAM_128:  return cString::sprintf("QAM 128");
-    case QAM_256:  return cString::sprintf("QAM 256");
-    case QAM_AUTO: return cString::sprintf("QAM %s", trVDR("auto"));
-    }
-  return cString::sprintf("---");
+  return cString::sprintf("%s", getUserString(value, ModulationValues));
+}
+
+cString getSystem(int value)
+{
+  return cString::sprintf("%s", getUserString(value, SystemValues));
+}
+
+cString getRollOff(int value)
+{
+  return cString::sprintf("%s", getUserString(value, RollOffValues));
 }
 
 cString getResolution(int width, int height, int scan)
@@ -403,6 +392,7 @@ cString getAspectRatio(int value)
     case VIDEO_ASPECT_RATIO_160_99:   return cString::sprintf("160:99");
     case VIDEO_ASPECT_RATIO_3_2:      return cString::sprintf("3:2");
     case VIDEO_ASPECT_RATIO_2_1:      return cString::sprintf("2:1");
+    default:                          break;
     }
   return cString::sprintf("---");
 }
@@ -417,6 +407,7 @@ cString getVideoFormat(int value)
     case VIDEO_FORMAT_NTSC:      return cString::sprintf("%s", tr("NTSC"));
     case VIDEO_FORMAT_SECAM:     return cString::sprintf("%s", tr("SECAM"));
     case VIDEO_FORMAT_MAC:       return cString::sprintf("%s", tr("MAC"));
+    default:                     break;
     }
   return cString::sprintf("---");
 }
@@ -439,6 +430,7 @@ cString getAC3BitStreamMode(int value, int coding)
     case AUDIO_BITSTREAM_MODE_C:      return cString::sprintf("%s", tr("Commentary (C)"));
     case AUDIO_BITSTREAM_MODE_E:      return cString::sprintf("%s", tr("Emergency (E)"));
     case AUDIO_BITSTREAM_MODE_VO_KAR: return cString::sprintf("%s", (coding == 1) ? tr("Voice Over (VO)") : tr("Karaoke"));
+    default:                          break;
     }
   return cString::sprintf("---");
 }
@@ -455,6 +447,7 @@ cString getAC3AudioCodingMode(int value, int stream)
        case AUDIO_CODING_MODE_3_1: return cString::sprintf("3/1 - %s, %s, %s, %s", tr("L"), tr("C"), tr("R"), tr("S"));
        case AUDIO_CODING_MODE_2_2: return cString::sprintf("2/2 - %s, %s, %s, %s", tr("L"), tr("R"), tr("SL"), tr("SR"));
        case AUDIO_CODING_MODE_3_2: return cString::sprintf("3/2 - %s, %s, %s, %s, %s", tr("L"), tr("C"), tr("R"), tr("SL"), tr("SR"));
+       default:                    break;
        }
      }
   return cString::sprintf("---");
@@ -467,6 +460,7 @@ cString getAC3CenterMixLevel(int value)
     case AUDIO_CENTER_MIX_LEVEL_MINUS_4_5dB: return cString::sprintf("-4.5 %s", tr("dB"));
     case AUDIO_CENTER_MIX_LEVEL_MINUS_6dB:   return cString::sprintf("-6.0 %s", tr("dB"));
     case AUDIO_CENTER_MIX_LEVEL_RESERVED:    return cString::sprintf("%s", tr("reserved"));
+    default:                                 break;
     }
   return cString::sprintf("---");
 }
@@ -478,6 +472,7 @@ cString getAC3SurroundMixLevel(int value)
     case AUDIO_SURROUND_MIX_LEVEL_MINUS_6dB: return cString::sprintf("-6 %s", tr("dB"));
     case AUDIO_SURROUND_MIX_LEVEL_0_dB:      return cString::sprintf("0 %s", tr("dB"));
     case AUDIO_SURROUND_MIX_LEVEL_RESERVED:  return cString::sprintf("%s", tr("reserved"));
+    default:                                 break;
     }
   return cString::sprintf("---");
 }
@@ -489,6 +484,7 @@ cString getAC3DolbySurroundMode(int value)
     case AUDIO_DOLBY_SURROUND_MODE_NOT_DOLBYSURROUND: return cString::sprintf("%s", trVDR("no"));
     case AUDIO_DOLBY_SURROUND_MODE_DOLBYSURROUND:     return cString::sprintf("%s", trVDR("yes"));
     case AUDIO_DOLBY_SURROUND_MODE_RESERVED:          return cString::sprintf("%s", tr("reserved"));
+    default:                                          break;
     }
   return cString::sprintf("---");
 }
@@ -511,6 +507,7 @@ cString getAudioSamplingFreq(int value)
   switch (value) {
     case AUDIO_SAMPLING_FREQUENCY_INVALID:  return cString::sprintf("---");
     case AUDIO_SAMPLING_FREQUENCY_RESERVED: return cString::sprintf("%s", tr("reserved"));
+    default:                                break;
     }
   return cString::sprintf("%d %s", value, tr("Hz"));
 }
@@ -521,6 +518,7 @@ cString getAudioBitrate(double value, double stream)
     case AUDIO_BITRATE_INVALID:  return cString::sprintf("---");
     case AUDIO_BITRATE_RESERVED: return cString::sprintf("%s (%s)", tr("reserved"), *getBitrateKbits(value));
     case AUDIO_BITRATE_FREE:     return cString::sprintf("%s (%s)", tr("free"), *getBitrateKbits(value));
+    default:                     break;
     }
   return cString::sprintf("%s (%s)", *getBitrateKbits(stream), *getBitrateKbits(value));
 }
@@ -539,7 +537,89 @@ cString getBitrateMbits(double value)
 
 cString getBitrateKbits(double value)
 {
-  if (value > 0) 
+  if (value > 0)
      return cString::sprintf("%.0f %s", value / 1000.0, tr("kbit/s"));
   return cString::sprintf("---");
 }
+
+cBitStream::cBitStream(const uint8_t *buf, const int len)
+: data(buf),
+  count(len),
+  index(0)
+{
+}
+
+cBitStream::~cBitStream()
+{
+}
+
+int cBitStream::getBit()
+{
+  if (index >= count)
+     return (1); // -> no infinite colomb's ...
+
+  int r = (data[index >> 3] >> (7 - (index & 7))) & 1;
+  ++index;
+
+  return (r);
+}
+
+uint32_t cBitStream::getBits(uint32_t n)
+{
+  uint32_t r = 0;
+
+  while (n--)
+    r = (r | (getBit() << n));
+
+  return (r);
+}
+
+void cBitStream::skipBits(uint32_t n)
+{
+  index += n;
+}
+
+uint32_t cBitStream::getUeGolomb()
+{
+  int n = 0;
+
+  while (!getBit() && (n < 32))
+    n++;
+
+  return (n ? ((1 << n) - 1) + getBits(n) : 0);
+}
+
+int32_t cBitStream::getSeGolomb()
+{
+  uint32_t r = getUeGolomb() + 1;
+
+  return ((r & 1) ? -(r >> 1) : (r >> 1));
+}
+
+void cBitStream::skipGolomb()
+{
+  int n = 0;
+
+  while (!getBit() && (n < 32))
+    n++;
+
+  skipBits(n);
+}
+
+void cBitStream::skipUeGolomb()
+{
+  skipGolomb();
+}
+
+void cBitStream::skipSeGolomb()
+{
+  skipGolomb();
+}
+
+void cBitStream::byteAlign()
+{
+  int n = index % 8;
+
+  if (n > 0)
+     skipBits(8 - n);
+}
