@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: device.c 1.141 2007/06/16 09:31:32 kls Exp $
+ * $Id: device.c 2.37 2010/06/03 13:35:02 kls Exp $
  */
 
 #include "device.h"
@@ -18,138 +18,67 @@
 #include "receiver.h"
 #include "status.h"
 #include "transfer.h"
+#include "vdrttxtsubshooks.h"
 
-// --- cPesAssembler ---------------------------------------------------------
+// --- cLiveSubtitle ---------------------------------------------------------
 
-class cPesAssembler {
-private:
-  uchar *data;
-  uint32_t tag;
-  int length;
-  int size;
-  bool Realloc(int Size);
+class cLiveSubtitle : public cReceiver {
+protected:
+  virtual void Receive(uchar *Data, int Length);
 public:
-  cPesAssembler(void);
-  ~cPesAssembler();
-  int ExpectedLength(void) { return PacketSize(data); }
-  static int PacketSize(const uchar *data);
-  int Length(void) { return length; }
-  const uchar *Data(void) { return data; } // only valid if Length() >= 4
-  void Reset(void);
-  void Put(uchar c);
-  void Put(const uchar *Data, int Length);
-  bool IsPes(void);
+  cLiveSubtitle(int SPid);
+  virtual ~cLiveSubtitle();
   };
 
-cPesAssembler::cPesAssembler(void)
+cLiveSubtitle::cLiveSubtitle(int SPid)
 {
-  data = NULL;
-  size = 0;
-  Reset();
+  AddPid(SPid);
 }
 
-cPesAssembler::~cPesAssembler()
+cLiveSubtitle::~cLiveSubtitle()
 {
-  free(data);
+  cReceiver::Detach();
 }
 
-void cPesAssembler::Reset(void)
+void cLiveSubtitle::Receive(uchar *Data, int Length)
 {
-  tag = 0xFFFFFFFF;
-  length = 0;
+  cDevice::PrimaryDevice()->PlayTs(Data, Length);
 }
 
-bool cPesAssembler::Realloc(int Size)
+// --- cDeviceHook -----------------------------------------------------------
+
+cDeviceHook::cDeviceHook(void)
+{
+  cDevice::deviceHooks.Add(this);
+}
+
+bool cDeviceHook::DeviceProvidesTransponder(const cDevice *Device, const cChannel *Channel) const
 {
-  if (Size > size) {
-     size = max(Size, 2048);
-     data = (uchar *)realloc(data, size);
-     if (!data) {
-        esyslog("ERROR: can't allocate memory for PES assembler");
-        length = 0;
-        size = 0;
-        return false;
-        }
-     }
   return true;
 }
 
-void cPesAssembler::Put(uchar c)
-{
-  if (length < 4) {
-     tag = (tag << 8) | c;
-     if ((tag & 0xFFFFFF00) == 0x00000100) {
-        if (Realloc(4)) {
-           *(uint32_t *)data = htonl(tag);
-           length = 4;
-           }
-        }
-     else if (length < 3)
-        length++;
-     }
-  else if (Realloc(length + 1))
-     data[length++] = c;
-}
-
-void cPesAssembler::Put(const uchar *Data, int Length)
-{
-  while (length < 4 && Length > 0) {
-        Put(*Data++);
-        Length--;
-        }
-  if (Length && Realloc(length + Length)) {
-     memcpy(data + length, Data, Length);
-     length += Length;
-     }
-}
-
-int cPesAssembler::PacketSize(const uchar *data)
-{
-  // we need atleast 6 bytes of data here !!!
-  switch (data[3]) {
-    default:
-    case 0x00 ... 0xB8: // video stream start codes
-    case 0xB9: // Program end
-    case 0xBC: // Programm stream map
-    case 0xF0 ... 0xFF: // reserved
-         return 6;
-
-    case 0xBA: // Pack header
-         if ((data[4] & 0xC0) == 0x40) // MPEG2
-            return 14;
-         // to be absolutely correct we would have to add the stuffing bytes
-         // as well, but at this point we only may have 6 bytes of data avail-
-         // able. So it's up to the higher level to resync...
-         //return 14 + (data[13] & 0x07); // add stuffing bytes
-         else // MPEG1
-            return 12;
-
-    case 0xBB: // System header
-    case 0xBD: // Private stream1
-    case 0xBE: // Padding stream
-    case 0xBF: // Private stream2 (navigation data)
-    case 0xC0 ... 0xCF: // all the rest (the real packets)
-    case 0xD0 ... 0xDF:
-    case 0xE0 ... 0xEF:
-         return 6 + data[4] * 256 + data[5];
-    }
-}
-
 // --- cDevice ---------------------------------------------------------------
 
 // The default priority for non-primary devices:
 #define DEFAULTPRIORITY  -1
 
+// The minimum number of unknown PS1 packets to consider this a "pre 1.3.19 private stream":
+#define MIN_PRE_1_3_19_PRIVATESTREAM 10
+
 int cDevice::numDevices = 0;
 int cDevice::useDevice = 0;
 int cDevice::nextCardIndex = 0;
 int cDevice::currentChannel = 1;
 cDevice *cDevice::device[MAXDEVICES] = { NULL };
 cDevice *cDevice::primaryDevice = NULL;
+cDevice *cDevice::avoidDevice = NULL;
+cList<cDeviceHook> cDevice::deviceHooks;
 
 cDevice::cDevice(void)
+:patPmtParser(true)
 {
   cardIndex = nextCardIndex++;
+  dsyslog("new device number %d", CardIndex() + 1);
 
   SetDescription("receiver on device %d", CardIndex() + 1);
 
@@ -168,10 +97,14 @@ cDevice::cDevice(void)
   startScrambleDetection = 0;
 
   player = NULL;
-  pesAssembler = new cPesAssembler;
+  isPlayingVideo = false;
   ClrAvailableTracks();
   currentAudioTrack = ttNone;
   currentAudioTrackMissingCount = 0;
+  currentSubtitleTrack = ttNone;
+  liveSubtitle = NULL;
+  dvbSubtitleConverter = NULL;
+  autoSelectPreferredSubtitleLanguage = true;
 
   for (int i = 0; i < MAXRECEIVERS; i++)
       receiver[i] = NULL;
@@ -186,12 +119,8 @@ cDevice::~cDevice()
 {
   Detach(player);
   DetachAllReceivers();
-  delete nitFilter;
-  delete sdtFilter;
-  delete patFilter;
-  delete eitFilter;
-  delete sectionHandler;
-  delete pesAssembler;
+  delete liveSubtitle;
+  delete dvbSubtitleConverter;
 }
 
 bool cDevice::WaitForAllDevicesReady(int Timeout)
@@ -224,7 +153,7 @@ int cDevice::NextCardIndex(int n)
         esyslog("ERROR: nextCardIndex too big (%d)", nextCardIndex);
      }
   else if (n < 0)
-     esyslog("ERROR: invalid value in IncCardIndex(%d)", n);
+     esyslog("ERROR: invalid value in nextCardIndex(%d)", n);
   return nextCardIndex;
 }
 
@@ -239,6 +168,10 @@ int cDevice::DeviceNumber(void) const
 
 void cDevice::MakePrimaryDevice(bool On)
 {
+  if (!On) {
+     DELETENULL(liveSubtitle);
+     DELETENULL(dvbSubtitleConverter);
+     }
 }
 
 bool cDevice::SetPrimaryDevice(int n)
@@ -251,6 +184,7 @@ bool cDevice::SetPrimaryDevice(int n)
      primaryDevice = device[n];
      primaryDevice->MakePrimaryDevice(true);
      primaryDevice->SetVideoFormat(Setup.VideoFormat);
+     primaryDevice->SetVolumeDevice(Setup.CurrentVolume);
      return true;
      }
   esyslog("ERROR: invalid primary device number: %d", n + 1);
@@ -280,8 +214,25 @@ cDevice *cDevice::GetDevice(int Index)
   return (0 <= Index && Index < numDevices) ? device[Index] : NULL;
 }
 
+static int GetClippedNumProvidedSystems(int AvailableBits, cDevice *Device)
+{
+  int MaxNumProvidedSystems = (1 << AvailableBits) - 1;
+  int NumProvidedSystems = Device->NumProvidedSystems();
+  if (NumProvidedSystems > MaxNumProvidedSystems) {
+     esyslog("ERROR: device %d supports %d modulation systems but cDevice::GetDevice() currently only supports %d delivery systems which should be fixed", Device->CardIndex() + 1, NumProvidedSystems, MaxNumProvidedSystems);
+     NumProvidedSystems = MaxNumProvidedSystems;
+     }
+  else if (NumProvidedSystems <= 0) {
+     esyslog("ERROR: device %d reported an invalid number (%d) of supported delivery systems - assuming 1", Device->CardIndex() + 1, NumProvidedSystems);
+     NumProvidedSystems = 1;
+     }
+  return NumProvidedSystems;
+}
+
 cDevice *cDevice::GetDevice(const cChannel *Channel, int Priority, bool LiveView)
 {
+  cDevice *AvoidDevice = avoidDevice;
+  avoidDevice = NULL;
   // Collect the current priorities of all CAM slots that can decrypt the channel:
   int NumCamSlots = CamSlots.Count();
   int SlotPriority[NumCamSlots];
@@ -311,6 +262,8 @@ cDevice *cDevice::GetDevice(const cChannel *Channel, int Priority, bool LiveView
       if (NumUsableSlots && SlotPriority[j] > MAXPRIORITY)
          continue; // there is no CAM available in this slot
       for (int i = 0; i < numDevices; i++) {
+          if (device[i] == AvoidDevice)
+             continue; // this device shall be temporarily avoided
           if (Channel->Ca() && Channel->Ca() <= CA_DVB_MAX && Channel->Ca() != device[i]->CardIndex() + 1)
              continue; // a specific card was requested, but not this one
           if (NumUsableSlots && !CamSlots.Get(j)->Assign(device[i], true))
@@ -329,6 +282,7 @@ cDevice *cDevice::GetDevice(const cChannel *Channel, int Priority, bool LiveView
              imp <<= 1; imp |= LiveView ? !device[i]->IsPrimaryDevice() || ndr : 0;                                  // prefer the primary device for live viewing if we don't need to detach existing receivers
              imp <<= 1; imp |= !device[i]->Receiving() && (device[i] != cTransferControl::ReceiverDevice() || device[i]->IsPrimaryDevice()) || ndr; // use receiving devices if we don't need to detach existing receivers, but avoid primary device in local transfer mode
              imp <<= 1; imp |= device[i]->Receiving();                                                               // avoid devices that are receiving
+             imp <<= 4; imp |= GetClippedNumProvidedSystems(4, device[i]) - 1;                                       // avoid cards which support multiple delivery systems
              imp <<= 1; imp |= device[i] == cTransferControl::ReceiverDevice();                                      // avoid the Transfer Mode receiver device
              imp <<= 8; imp |= min(max(device[i]->Priority() + MAXPRIORITY, 0), 0xFF);                               // use the device with the lowest priority (+MAXPRIORITY to assure that values -99..99 can be used)
              imp <<= 8; imp |= min(max((NumUsableSlots ? SlotPriority[j] : 0) + MAXPRIORITY, 0), 0xFF);              // use the CAM slot with the lowest priority (+MAXPRIORITY to assure that values -99..99 can be used)
@@ -436,6 +390,7 @@ void cDevice::SetVideoDisplayFormat(eVideoDisplayFormat VideoDisplayFormat)
                case vdfCenterCutOut:
                     spuDecoder->setScaleMode(cSpuDecoder::eSpuNormal);
                     break;
+               default: esyslog("ERROR: invalid value for VideoDisplayFormat '%d'", VideoDisplayFormat);
                }
         }
      }
@@ -450,7 +405,21 @@ eVideoSystem cDevice::GetVideoSystem(void)
   return vsPAL;
 }
 
-//#define PRINTPIDS(s) { char b[500]; char *q = b; q += sprintf(q, "%d %s ", CardIndex(), s); for (int i = 0; i < MAXPIDHANDLES; i++) q += sprintf(q, " %s%4d %d", i == ptOther ? "* " : "", pidHandles[i].pid, pidHandles[i].used); dsyslog(b); }
+void cDevice::GetVideoSize(int &Width, int &Height, double &VideoAspect)
+{
+  Width = 0;
+  Height = 0;
+  VideoAspect = 1.0;
+}
+
+void cDevice::GetOsdSize(int &Width, int &Height, double &PixelAspect)
+{
+  Width = 720;
+  Height = 480;
+  PixelAspect = 1.0;
+}
+
+//#define PRINTPIDS(s) { char b[500]; char *q = b; q += sprintf(q, "%d %s ", CardIndex(), s); for (int i = 0; i < MAXPIDHANDLES; i++) q += sprintf(q, " %s%4d %d", i == ptOther ? "* " : "", pidHandles[i].pid, pidHandles[i].used); dsyslog("%s", b); }
 #define PRINTPIDS(s)
 
 bool cDevice::HasPid(int Pid) const
@@ -462,7 +431,7 @@ bool cDevice::HasPid(int Pid) const
   return false;
 }
 
-bool cDevice::AddPid(int Pid, ePidType PidType)
+bool cDevice::AddPid(int Pid, ePidType PidType, int StreamType)
 {
   if (Pid || PidType == ptPcr) {
      int n = -1;
@@ -509,6 +478,7 @@ bool cDevice::AddPid(int Pid, ePidType PidType)
         }
      if (n >= 0) {
         pidHandles[n].pid = Pid;
+        pidHandles[n].streamType = StreamType;
         pidHandles[n].used = 1;
         PRINTPIDS("C");
         if (!SetPid(&pidHandles[n], n, true)) {
@@ -571,11 +541,32 @@ void cDevice::StartSectionHandler(void)
      }
 }
 
+void cDevice::StopSectionHandler(void)
+{
+  if (sectionHandler) {
+     delete nitFilter;
+     delete sdtFilter;
+     delete patFilter;
+     delete eitFilter;
+     delete sectionHandler;
+     nitFilter = NULL;
+     sdtFilter = NULL;
+     patFilter = NULL;
+     eitFilter = NULL;
+     sectionHandler = NULL;
+     }
+}
+
 int cDevice::OpenFilter(u_short Pid, u_char Tid, u_char Mask)
 {
   return -1;
 }
 
+void cDevice::CloseFilter(int Handle)
+{
+  close(Handle);
+}
+
 void cDevice::AttachFilter(cFilter *Filter)
 {
   if (sectionHandler)
@@ -593,6 +584,17 @@ bool cDevice::ProvidesSource(int Source) const
   return false;
 }
 
+bool cDevice::DeviceHooksProvidesTransponder(const cChannel *Channel) const
+{
+  cDeviceHook *Hook = deviceHooks.First();
+  while (Hook) {
+        if (!Hook->DeviceProvidesTransponder(this, Channel))
+           return false;
+        Hook = deviceHooks.Next(Hook);
+        }
+  return true;
+}
+
 bool cDevice::ProvidesTransponder(const cChannel *Channel) const
 {
   return false;
@@ -612,6 +614,16 @@ bool cDevice::ProvidesChannel(const cChannel *Channel, int Priority, bool *Needs
   return false;
 }
 
+int cDevice::NumProvidedSystems(void) const
+{
+  return 0;
+}
+
+const cChannel *cDevice::GetCurrentlyTunedTransponder(void) const
+{
+  return NULL;
+}
+
 bool cDevice::IsTunedToTransponder(const cChannel *Channel)
 {
   return false;
@@ -636,6 +648,7 @@ bool cDevice::SwitchChannel(const cChannel *Channel, bool LiveView)
         case scrNoTransfer:   Skins.Message(mtError, tr("Can't start Transfer Mode!"));
                               return false;
         case scrFailed:       break; // loop will retry
+        default:              esyslog("ERROR: invalid return value from SetChannel");
         }
       esyslog("retrying");
       }
@@ -674,8 +687,11 @@ bool cDevice::SwitchChannel(int Direction)
 
 eSetChannelResult cDevice::SetChannel(const cChannel *Channel, bool LiveView)
 {
-  if (LiveView)
+  if (LiveView) {
      StopReplay();
+     DELETENULL(liveSubtitle);
+     DELETENULL(dvbSubtitleConverter);
+     }
 
   cDevice *Device = (LiveView && IsPrimaryDevice()) ? GetDevice(Channel, 0, LiveView) : this;
 
@@ -690,7 +706,7 @@ eSetChannelResult cDevice::SetChannel(const cChannel *Channel, bool LiveView)
      if (Device && CanReplay()) {
         cStatus::MsgChannelSwitch(this, 0); // only report status if we are actually going to switch the channel
         if (Device->SetChannel(Channel, false) == scrOk) // calling SetChannel() directly, not SwitchChannel()!
-           cControl::Launch(new cTransferControl(Device, Channel->GetChannelID(), Channel->Vpid(), Channel->Apids(), Channel->Dpids(), Channel->Spids()));
+           cControl::Launch(new cTransferControl(Device, Channel));
         else
            Result = scrNoTransfer;
         }
@@ -735,8 +751,11 @@ eSetChannelResult cDevice::SetChannel(const cChannel *Channel, bool LiveView)
            for (int i = 0; i < MAXDPIDS; i++)
                SetAvailableTrack(ttDolby, i, Channel->Dpid(i), Channel->Dlang(i));
            }
+        for (int i = 0; i < MAXSPIDS; i++)
+            SetAvailableTrack(ttSubtitle, i, Channel->Spid(i), Channel->Slang(i));
         if (!NeedsTransferMode)
            EnsureAudioTrack(true);
+        EnsureSubtitleTrack();
         }
      cStatus::MsgChannelSwitch(this, Channel->Number()); // only report status if channel switch successfull
      }
@@ -789,6 +808,10 @@ void cDevice::SetAudioTrackDevice(eTrackType Type)
 {
 }
 
+void cDevice::SetSubtitleTrackDevice(eTrackType Type)
+{
+}
+
 bool cDevice::ToggleMute(void)
 {
   int OldVolume = volume;
@@ -844,10 +867,11 @@ void cDevice::ClrAvailableTracks(bool DescriptionsOnly, bool IdsOnly)
         }
      else
         memset(availableTracks, 0, sizeof(availableTracks));
-     pre_1_3_19_PrivateStream = false;
+     pre_1_3_19_PrivateStream = 0;
      SetAudioChannel(0); // fall back to stereo
      currentAudioTrackMissingCount = 0;
      currentAudioTrack = ttNone;
+     currentSubtitleTrack = ttNone;
      }
 }
 
@@ -855,18 +879,23 @@ bool cDevice::SetAvailableTrack(eTrackType Type, int Index, uint16_t Id, const c
 {
   eTrackType t = eTrackType(Type + Index);
   if (Type == ttAudio && IS_AUDIO_TRACK(t) ||
-      Type == ttDolby && IS_DOLBY_TRACK(t)) {
+      Type == ttDolby && IS_DOLBY_TRACK(t) ||
+      Type == ttSubtitle && IS_SUBTITLE_TRACK(t)) {
      if (Language)
         strn0cpy(availableTracks[t].language, Language, sizeof(availableTracks[t].language));
      if (Description)
         Utf8Strn0Cpy(availableTracks[t].description, Description, sizeof(availableTracks[t].description));
      if (Id) {
         availableTracks[t].id = Id; // setting 'id' last to avoid the need for extensive locking
-        int numAudioTracks = NumAudioTracks();
-        if (!availableTracks[currentAudioTrack].id && numAudioTracks && currentAudioTrackMissingCount++ > numAudioTracks * 10)
-           EnsureAudioTrack();
-        else if (t == currentAudioTrack)
-           currentAudioTrackMissingCount = 0;
+        if (Type == ttAudio || Type == ttDolby) {
+           int numAudioTracks = NumAudioTracks();
+           if (!availableTracks[currentAudioTrack].id && numAudioTracks && currentAudioTrackMissingCount++ > numAudioTracks * 10)
+              EnsureAudioTrack();
+           else if (t == currentAudioTrack)
+              currentAudioTrackMissingCount = 0;
+           }
+        else if (Type == ttSubtitle && autoSelectPreferredSubtitleLanguage)
+           EnsureSubtitleTrack();
         }
      return true;
      }
@@ -880,19 +909,29 @@ const tTrackId *cDevice::GetTrack(eTrackType Type)
   return (ttNone < Type && Type < ttMaxTrackTypes) ? &availableTracks[Type] : NULL;
 }
 
-int cDevice::NumAudioTracks(void) const
+int cDevice::NumTracks(eTrackType FirstTrack, eTrackType LastTrack) const
 {
   int n = 0;
-  for (int i = ttAudioFirst; i <= ttDolbyLast; i++) {
+  for (int i = FirstTrack; i <= LastTrack; i++) {
       if (availableTracks[i].id)
          n++;
       }
   return n;
 }
 
+int cDevice::NumAudioTracks(void) const
+{
+  return NumTracks(ttAudioFirst, ttDolbyLast);
+}
+
+int cDevice::NumSubtitleTracks(void) const
+{
+  return NumTracks(ttSubtitleFirst, ttSubtitleLast);
+}
+
 bool cDevice::SetCurrentAudioTrack(eTrackType Type)
 {
-  if (ttNone < Type && Type < ttDolbyLast) {
+  if (ttNone < Type && Type <= ttDolbyLast) {
      cMutexLock MutexLock(&mutexCurrentAudioTrack);
      if (IS_DOLBY_TRACK(Type))
         SetDigitalAudioDevice(true);
@@ -908,6 +947,34 @@ bool cDevice::SetCurrentAudioTrack(eTrackType Type)
   return false;
 }
 
+bool cDevice::SetCurrentSubtitleTrack(eTrackType Type, bool Manual)
+{
+  if (Type == ttNone || IS_SUBTITLE_TRACK(Type)) {
+     currentSubtitleTrack = Type;
+     autoSelectPreferredSubtitleLanguage = !Manual;
+     if (dvbSubtitleConverter)
+        dvbSubtitleConverter->Reset();
+     if (Type == ttNone && dvbSubtitleConverter) {
+        cMutexLock MutexLock(&mutexCurrentSubtitleTrack);
+        DELETENULL(dvbSubtitleConverter);
+        }
+     DELETENULL(liveSubtitle);
+     if (player)
+        player->SetSubtitleTrack(currentSubtitleTrack, GetTrack(currentSubtitleTrack));
+     else
+        SetSubtitleTrackDevice(currentSubtitleTrack);
+     if (currentSubtitleTrack != ttNone && !Replaying() && !Transferring()) {
+        const tTrackId *TrackId = GetTrack(currentSubtitleTrack);
+        if (TrackId && TrackId->id) {
+           liveSubtitle = new cLiveSubtitle(TrackId->id);
+           AttachReceiver(liveSubtitle);
+           }
+        }
+     return true;
+     }
+  return false;
+}
+
 void cDevice::EnsureAudioTrack(bool Force)
 {
   if (Force || !availableTracks[currentAudioTrack].id) {
@@ -939,6 +1006,26 @@ void cDevice::EnsureAudioTrack(bool Force)
      }
 }
 
+void cDevice::EnsureSubtitleTrack(void)
+{
+  if (Setup.DisplaySubtitles) {
+     eTrackType PreferredTrack = ttNone;
+     int LanguagePreference = INT_MAX; // higher than the maximum possible value
+     for (int i = ttSubtitleFirst; i <= ttSubtitleLast; i++) {
+         const tTrackId *TrackId = GetTrack(eTrackType(i));
+         if (TrackId && TrackId->id && (I18nIsPreferredLanguage(Setup.SubtitleLanguages, TrackId->language, LanguagePreference) ||
+             ((i == ttSubtitleFirst + 8) && !(*TrackId->language) && (LanguagePreference == INT_MAX))))
+            PreferredTrack = eTrackType(i);
+         }
+     // Make sure we're set to an available subtitle track:
+     const tTrackId *Track = GetTrack(GetCurrentSubtitleTrack());
+     if (!Track || !Track->id || PreferredTrack != GetCurrentSubtitleTrack())
+        SetCurrentSubtitleTrack(PreferredTrack);
+     }
+  else
+     SetCurrentSubtitleTrack(ttNone);
+}
+
 bool cDevice::CanReplay(void) const
 {
   return HasDecoder();
@@ -961,16 +1048,22 @@ void cDevice::TrickSpeed(int Speed)
 void cDevice::Clear(void)
 {
   Audios.ClearAudio();
+  if (dvbSubtitleConverter)
+     dvbSubtitleConverter->Reset();
 }
 
 void cDevice::Play(void)
 {
   Audios.MuteAudio(mute);
+  if (dvbSubtitleConverter)
+     dvbSubtitleConverter->Freeze(false);
 }
 
 void cDevice::Freeze(void)
 {
   Audios.MuteAudio(true);
+  if (dvbSubtitleConverter)
+     dvbSubtitleConverter->Freeze(true);
 }
 
 void cDevice::Mute(void)
@@ -980,6 +1073,47 @@ void cDevice::Mute(void)
 
 void cDevice::StillPicture(const uchar *Data, int Length)
 {
+  if (Data[0] == 0x47) {
+     // TS data
+     cTsToPes TsToPes;
+     uchar *buf = NULL;
+     int Size = 0;
+     while (Length >= TS_SIZE) {
+           int Pid = TsPid(Data);
+           if (Pid == 0)
+              patPmtParser.ParsePat(Data, TS_SIZE);
+           else if (Pid == patPmtParser.PmtPid())
+              patPmtParser.ParsePmt(Data, TS_SIZE);
+           else if (Pid == patPmtParser.Vpid()) {
+              if (TsPayloadStart(Data)) {
+                 int l;
+                 while (const uchar *p = TsToPes.GetPes(l)) {
+                       int Offset = Size;
+                       Size += l;
+                       buf = (uchar *)realloc(buf, Size);
+                       if (!buf)
+                          return;
+                       memcpy(buf + Offset, p, l);
+                       }
+                 TsToPes.Reset();
+                 }
+              TsToPes.PutTs(Data, TS_SIZE);
+              }
+           Length -= TS_SIZE;
+           Data += TS_SIZE;
+           }
+     int l;
+     while (const uchar *p = TsToPes.GetPes(l)) {
+           int Offset = Size;
+           Size += l;
+           buf = (uchar *)realloc(buf, Size);
+           if (!buf)
+              return;
+           memcpy(buf + Offset, p, l);
+           }
+     StillPicture(buf, Size);
+     free(buf);
+     }
 }
 
 bool cDevice::Replaying(void) const
@@ -989,7 +1123,7 @@ bool cDevice::Replaying(void) const
 
 bool cDevice::Transferring(void) const
 {
-  return dynamic_cast<cTransfer *>(player) != NULL;
+  return ActualDevice() != PrimaryDevice();
 }
 
 bool cDevice::AttachPlayer(cPlayer *Player)
@@ -997,7 +1131,9 @@ bool cDevice::AttachPlayer(cPlayer *Player)
   if (CanReplay()) {
      if (player)
         Detach(player);
-     pesAssembler->Reset();
+     DELETENULL(liveSubtitle);
+     DELETENULL(dvbSubtitleConverter);
+     patPmtParser.Reset();
      player = Player;
      if (!Transferring())
         ClrAvailableTracks(false, true);
@@ -1016,9 +1152,15 @@ void cDevice::Detach(cPlayer *Player)
      player = NULL; // avoids recursive calls to Detach()
      p->Activate(false);
      p->device = NULL;
+     cMutexLock MutexLock(&mutexCurrentSubtitleTrack);
+     delete dvbSubtitleConverter;
+     dvbSubtitleConverter = NULL;
      SetPlayMode(pmNone);
      SetVideoDisplayFormat(eVideoDisplayFormat(Setup.VideoDisplayFormat));
+     PlayTs(NULL, 0);
+     patPmtParser.Reset();
      Audios.ClearAudio();
+     isPlayingVideo = false;
      }
 }
 
@@ -1051,6 +1193,13 @@ int cDevice::PlayAudio(const uchar *Data, int Length, uchar Id)
   return -1;
 }
 
+int cDevice::PlaySubtitle(const uchar *Data, int Length)
+{
+  if (!dvbSubtitleConverter)
+     dvbSubtitleConverter = new cDvbSubtitleConverter;
+  return dvbSubtitleConverter->ConvertFragments(Data, Length);
+}
+
 int cDevice::PlayPesPacket(const uchar *Data, int Length, bool VideoOnly)
 {
   cMutexLock MutexLock(&mutexCurrentAudioTrack);
@@ -1064,37 +1213,55 @@ int cDevice::PlayPesPacket(const uchar *Data, int Length, bool VideoOnly)
         switch (c) {
           case 0xBE:          // padding stream, needed for MPEG1
           case 0xE0 ... 0xEF: // video
+               isPlayingVideo = true;
                w = PlayVideo(Start, d);
                break;
           case 0xC0 ... 0xDF: // audio
                SetAvailableTrack(ttAudio, c - 0xC0, c);
-               if (!VideoOnly && c == availableTracks[currentAudioTrack].id) {
+               if ((!VideoOnly || HasIBPTrickSpeed()) && c == availableTracks[currentAudioTrack].id) {
                   w = PlayAudio(Start, d, c);
                   if (FirstLoop)
                      Audios.PlayAudio(Data, Length, c);
                   }
                break;
           case 0xBD: { // private stream 1
+               // EBU Teletext data, ETSI EN 300 472
+               // if PES data header length = 24 and data_identifier = 0x10..0x1F (EBU Data)
+               if (Data[8] == 0x24 && Data[45] >= 0x10 && Data[45] < 0x20) {
+                  cVDRTtxtsubsHookListener::Hook()->PlayerTeletextData((uint8_t*)Data, Length);
+                  break;
+                  }
+
                int PayloadOffset = Data[8] + 9;
+
+               // Compatibility mode for old subtitles plugin:
+               if ((Data[7] & 0x01) && (Data[PayloadOffset - 3] & 0x81) == 0x01 && Data[PayloadOffset - 2] == 0x81)
+                  PayloadOffset--;
+
                uchar SubStreamId = Data[PayloadOffset];
                uchar SubStreamType = SubStreamId & 0xF0;
                uchar SubStreamIndex = SubStreamId & 0x1F;
 
                // Compatibility mode for old VDR recordings, where 0xBD was only AC3:
-pre_1_3_19_PrivateStreamDeteced:
-               if (pre_1_3_19_PrivateStream) {
+pre_1_3_19_PrivateStreamDetected:
+               if (pre_1_3_19_PrivateStream > MIN_PRE_1_3_19_PRIVATESTREAM) {
                   SubStreamId = c;
                   SubStreamType = 0x80;
                   SubStreamIndex = 0;
                   }
+               else if (pre_1_3_19_PrivateStream)
+                  pre_1_3_19_PrivateStream--; // every known PS1 packet counts down towards 0 to recover from glitches...
                switch (SubStreamType) {
                  case 0x20: // SPU
                  case 0x30: // SPU
+                      SetAvailableTrack(ttSubtitle, SubStreamIndex, SubStreamId);
+                      if ((!VideoOnly || HasIBPTrickSpeed()) && currentSubtitleTrack != ttNone && SubStreamId == availableTracks[currentSubtitleTrack].id)
+                         w = PlaySubtitle(Start, d);
                       break;
                  case 0x80: // AC3 & DTS
                       if (Setup.UseDolbyDigital) {
                          SetAvailableTrack(ttDolby, SubStreamIndex, SubStreamId);
-                         if (!VideoOnly && SubStreamId == availableTracks[currentAudioTrack].id) {
+                         if ((!VideoOnly || HasIBPTrickSpeed()) && SubStreamId == availableTracks[currentAudioTrack].id) {
                             w = PlayAudio(Start, d, SubStreamId);
                             if (FirstLoop)
                                Audios.PlayAudio(Data, Length, SubStreamId);
@@ -1103,7 +1270,7 @@ pre_1_3_19_PrivateStreamDeteced:
                       break;
                  case 0xA0: // LPCM
                       SetAvailableTrack(ttAudio, SubStreamIndex, SubStreamId);
-                      if (!VideoOnly && SubStreamId == availableTracks[currentAudioTrack].id) {
+                      if ((!VideoOnly || HasIBPTrickSpeed()) && SubStreamId == availableTracks[currentAudioTrack].id) {
                          w = PlayAudio(Start, d, SubStreamId);
                          if (FirstLoop)
                             Audios.PlayAudio(Data, Length, SubStreamId);
@@ -1111,11 +1278,15 @@ pre_1_3_19_PrivateStreamDeteced:
                       break;
                  default:
                       // Compatibility mode for old VDR recordings, where 0xBD was only AC3:
-                      if (!pre_1_3_19_PrivateStream) {
-                         dsyslog("switching to pre 1.3.19 Dolby Digital compatibility mode");
-                         ClrAvailableTracks();
-                         pre_1_3_19_PrivateStream = true;
-                         goto pre_1_3_19_PrivateStreamDeteced;
+                      if (pre_1_3_19_PrivateStream <= MIN_PRE_1_3_19_PRIVATESTREAM) {
+                         dsyslog("unknown PS1 packet, substream id = %02X (counter is at %d)", SubStreamId, pre_1_3_19_PrivateStream);
+                         pre_1_3_19_PrivateStream += 2; // ...and every unknown PS1 packet counts up (the very first one counts twice, but that's ok)
+                         if (pre_1_3_19_PrivateStream > MIN_PRE_1_3_19_PRIVATESTREAM) {
+                            dsyslog("switching to pre 1.3.19 Dolby Digital compatibility mode - substream id = %02X", SubStreamId);
+                            ClrAvailableTracks();
+                            pre_1_3_19_PrivateStream = MIN_PRE_1_3_19_PRIVATESTREAM + 1;
+                            goto pre_1_3_19_PrivateStreamDetected;
+                            }
                          }
                  }
                }
@@ -1138,40 +1309,16 @@ pre_1_3_19_PrivateStreamDeteced:
 int cDevice::PlayPes(const uchar *Data, int Length, bool VideoOnly)
 {
   if (!Data) {
-     pesAssembler->Reset();
+     if (dvbSubtitleConverter)
+        dvbSubtitleConverter->Reset();
      return 0;
      }
-  int Result = 0;
-  if (pesAssembler->Length()) {
-     // Make sure we have a complete PES header:
-     while (pesAssembler->Length() < 6 && Length > 0) {
-           pesAssembler->Put(*Data++);
-           Length--;
-           Result++;
-           }
-     if (pesAssembler->Length() < 6)
-        return Result; // Still no complete PES header - wait for more
-     int l = pesAssembler->ExpectedLength();
-     int Rest = min(l - pesAssembler->Length(), Length);
-     pesAssembler->Put(Data, Rest);
-     Data += Rest;
-     Length -= Rest;
-     Result += Rest;
-     if (pesAssembler->Length() < l)
-        return Result; // Still no complete PES packet - wait for more
-     // Now pesAssembler contains one complete PES packet.
-     int w = PlayPesPacket(pesAssembler->Data(), pesAssembler->Length(), VideoOnly);
-     if (w > 0)
-        pesAssembler->Reset();
-     return Result > 0 ? Result : w < 0 ? w : 0;
-     }
   int i = 0;
   while (i <= Length - 6) {
         if (Data[i] == 0x00 && Data[i + 1] == 0x00 && Data[i + 2] == 0x01) {
-           int l = cPesAssembler::PacketSize(&Data[i]);
+           int l = PesLength(Data + i);
            if (i + l > Length) {
-              // Store incomplete PES packet for later completion:
-              pesAssembler->Put(Data + i, Length - i);
+              esyslog("ERROR: incomplete PES packet!");
               return Length;
               }
            int w = PlayPesPacket(Data + i, l, VideoOnly);
@@ -1184,10 +1331,140 @@ int cDevice::PlayPes(const uchar *Data, int Length, bool VideoOnly)
            i++;
         }
   if (i < Length)
-     pesAssembler->Put(Data + i, Length - i);
+     esyslog("ERROR: leftover PES data!");
   return Length;
 }
 
+int cDevice::PlayTsVideo(const uchar *Data, int Length)
+{
+  // Video PES has no explicit length, so we can only determine the end of
+  // a PES packet when the next TS packet that starts a payload comes in:
+  if (TsPayloadStart(Data)) {
+     int l;
+     while (const uchar *p = tsToPesVideo.GetPes(l)) {
+           int w = PlayVideo(p, l);
+           if (w <= 0) {
+              tsToPesVideo.SetRepeatLast();
+              return w;
+              }
+           }
+     tsToPesVideo.Reset();
+     }
+  tsToPesVideo.PutTs(Data, Length);
+  return Length;
+}
+
+int cDevice::PlayTsAudio(const uchar *Data, int Length)
+{
+  // Audio PES always has an explicit length and consists of single packets:
+  int l;
+  if (const uchar *p = tsToPesAudio.GetPes(l)) {
+     int w = PlayAudio(p, l, p[3]);
+     if (w <= 0) {
+        tsToPesAudio.SetRepeatLast();
+        return w;
+        }
+     tsToPesAudio.Reset();
+     }
+  tsToPesAudio.PutTs(Data, Length);
+  return Length;
+}
+
+int cDevice::PlayTsSubtitle(const uchar *Data, int Length)
+{
+  if (!dvbSubtitleConverter)
+     dvbSubtitleConverter = new cDvbSubtitleConverter;
+  tsToPesSubtitle.PutTs(Data, Length);
+  int l;
+  if (const uchar *p = tsToPesSubtitle.GetPes(l)) {
+     dvbSubtitleConverter->Convert(p, l);
+     tsToPesSubtitle.Reset();
+     }
+  return Length;
+}
+
+//TODO detect and report continuity errors?
+int cDevice::PlayTs(const uchar *Data, int Length, bool VideoOnly)
+{
+  int Played = 0;
+  if (Data == NULL) {
+     tsToPesVideo.Reset();
+     tsToPesAudio.Reset();
+     tsToPesSubtitle.Reset();
+     tsToPesTeletext.Reset();
+     }
+  else if (Length < TS_SIZE) {
+     esyslog("ERROR: skipped %d bytes of TS fragment", Length);
+     return Length;
+     }
+  else {
+     cMutexLock MutexLock(&mutexCurrentAudioTrack);
+     while (Length >= TS_SIZE) {
+           if (Data[0] != TS_SYNC_BYTE) {
+              int Skipped = 1;
+              while (Skipped < Length && (Data[Skipped] != TS_SYNC_BYTE || Length - Skipped > TS_SIZE && Data[Skipped + TS_SIZE] != TS_SYNC_BYTE))
+                    Skipped++;
+              esyslog("ERROR: skipped %d bytes to sync on start of TS packet", Skipped);
+              return Played + Skipped;
+              }
+           int Pid = TsPid(Data);
+           if (TsHasPayload(Data)) { // silently ignore TS packets w/o payload
+              int PayloadOffset = TsPayloadOffset(Data);
+              if (PayloadOffset < TS_SIZE) {
+                 if (Pid == 0)
+                    patPmtParser.ParsePat(Data, TS_SIZE);
+                 else if (Pid == patPmtParser.PmtPid())
+                    patPmtParser.ParsePmt(Data, TS_SIZE);
+                 else if (Pid == patPmtParser.Vpid()) {
+                    isPlayingVideo = true;
+                    int w = PlayTsVideo(Data, TS_SIZE);
+                    if (w < 0)
+                       return Played ? Played : w;
+                    if (w == 0)
+                       break;
+                    }
+                 else if (Pid == availableTracks[currentAudioTrack].id) {
+                    if (!VideoOnly || HasIBPTrickSpeed()) {
+                       int w = PlayTsAudio(Data, TS_SIZE);
+                       if (w < 0)
+                          return Played ? Played : w;
+                       if (w == 0)
+                          break;
+                       Audios.PlayTsAudio(Data, TS_SIZE);
+                       }
+                    }
+                 else if (Pid == availableTracks[currentSubtitleTrack].id) {
+                    if (!VideoOnly || HasIBPTrickSpeed())
+                       PlayTsSubtitle(Data, TS_SIZE);
+                    }
+                 else if (Pid == patPmtParser.Tpid()) {
+                    if (!VideoOnly || HasIBPTrickSpeed()) {
+                       int l;
+                       tsToPesTeletext.PutTs(Data, Length);
+                       if (const uchar *p = tsToPesTeletext.GetPes(l)) {
+                          if ((l > 45) && (p[0] == 0x00) && (p[1] == 0x00) && (p[2] == 0x01) && (p[3] == 0xbd) && (p[8] == 0x24) && (p[45] >= 0x10) && (p[45] < 0x20))
+                             cVDRTtxtsubsHookListener::Hook()->PlayerTeletextData((uchar *)p, l, false, patPmtParser.TeletextSubtitlePages(), patPmtParser.TotalTeletextSubtitlePages());
+                          tsToPesTeletext.Reset();
+                          }
+                       }
+                    }
+                 }
+              }
+           else if (Pid == patPmtParser.Ppid()) {
+              int w = PlayTsVideo(Data, TS_SIZE);
+              if (w < 0)
+                 return Played ? Played : w;
+              if (w == 0)
+                 break;
+              }
+           Played += TS_SIZE;
+           Length -= TS_SIZE;
+           Data += TS_SIZE;
+           }
+     }
+  return Played;
+}
+
 int cDevice::Priority(void) const
 {
   int priority = IsPrimaryDevice() ? Setup.PrimaryLimit - 1 : DEFAULTPRIORITY;
@@ -1212,9 +1489,8 @@ bool cDevice::Receiving(bool CheckAny) const
   return false;
 }
 
-#define TS_SCRAMBLING_CONTROL  0xC0
 #define TS_SCRAMBLING_TIMEOUT     3 // seconds to wait until a TS becomes unscrambled
-#define TS_SCRAMBLING_TIME_OK    10 // seconds before a Channel/CAM combination is marked a known to decrypt
+#define TS_SCRAMBLING_TIME_OK    10 // seconds before a Channel/CAM combination is marked as known to decrypt
 
 void cDevice::Action(void)
 {
@@ -1224,7 +1500,7 @@ void cDevice::Action(void)
            uchar *b = NULL;
            if (GetTSPacket(b)) {
               if (b) {
-                 int Pid = (((uint16_t)b[1] & PID_MASK_HI) << 8) | b[2];
+                 int Pid = TsPid(b);
                  // Check whether the TS packets are scrambled:
                  bool DetachReceivers = false;
                  bool DescramblingOk = false;
