@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: device.h 1.82 2007/07/22 11:20:13 kls Exp $
+ * $Id: device.h 2.22 2010/04/05 09:51:29 kls Exp $
  */
 
 #ifndef __DEVICE_H
@@ -12,10 +12,12 @@
 
 #include "channels.h"
 #include "ci.h"
+#include "dvbsubtitle.h"
 #include "eit.h"
 #include "filter.h"
 #include "nit.h"
 #include "pat.h"
+#include "remux.h"
 #include "ringbuffer.h"
 #include "sdt.h"
 #include "sections.h"
@@ -29,10 +31,6 @@
 #define MAXVOLUME         255
 #define VOLUMEDELTA         5 // used to increase/decrease the volume
 
-#define TS_SIZE          188
-#define TS_SYNC_BYTE     0x47
-#define PID_MASK_HI      0x1F
-
 enum eSetChannelResult { scrOk, scrNotAvailable, scrNoTransfer, scrFailed };
 
 enum ePlayMode { pmNone,           // audio/video from decoder
@@ -70,16 +68,15 @@ enum eTrackType { ttNone,
                   ttDolby,
                   ttDolbyFirst = ttDolby,
                   ttDolbyLast  = ttDolbyFirst + 15, // MAXDPIDS - 1
-                  /* future...
                   ttSubtitle,
                   ttSubtitleFirst = ttSubtitle,
-                  ttSubtitleLast  = ttSubtitleFirst + 7, // MAXSPIDS - 1
-                  */
+                  ttSubtitleLast  = ttSubtitleFirst + 31, // MAXSPIDS - 1
                   ttMaxTrackTypes
                 };
 
 #define IS_AUDIO_TRACK(t) (ttAudioFirst <= (t) && (t) <= ttAudioLast)
 #define IS_DOLBY_TRACK(t) (ttDolbyFirst <= (t) && (t) <= ttDolbyLast)
+#define IS_SUBTITLE_TRACK(t) (ttSubtitleFirst <= (t) && (t) <= ttSubtitleLast)
 
 struct tTrackId {
   uint16_t id;                  // The PES packet id or the PID.
@@ -89,16 +86,29 @@ struct tTrackId {
 
 class cPlayer;
 class cReceiver;
-class cPesAssembler;
+class cLiveSubtitle;
+
+class cDeviceHook : public cListObject {
+public:
+  cDeviceHook(void);
+          ///< Creates a new device hook object.
+          ///< Do not delete this object - it will be automatically deleted when the
+          ///< program ends.
+  virtual bool DeviceProvidesTransponder(const cDevice *Device, const cChannel *Channel) const;
+          ///< Returns true if the given Device can provide the given Channel's transponder.
+  };
 
 /// The cDevice class is the base from which actual devices can be derived.
 
 class cDevice : public cThread {
+  friend class cLiveSubtitle;
+  friend class cDeviceHook;
 private:
   static int numDevices;
   static int useDevice;
   static cDevice *device[MAXDEVICES];
   static cDevice *primaryDevice;
+  static cDevice *avoidDevice;
 public:
   static int NumDevices(void) { return numDevices; }
          ///< Returns the total number of devices.
@@ -143,6 +153,9 @@ public:
          ///< this device/CAM combination will be skipped in the next call to
          ///< GetDevice().
          ///< See also ProvidesChannel().
+  static void SetAvoidDevice(cDevice *Device) { avoidDevice = Device; }
+         ///< Sets the given Device to be temporarily avoided in the next call to
+         ///< GetDevice(const cChannel, int, bool).
   static void Shutdown(void);
          ///< Closes down all devices.
          ///< Must be called at the end of the program.
@@ -174,6 +187,8 @@ protected:
          ///< anything the device needs to set up when it becomes the primary
          ///< device (On = true) or to shut down when it no longer is the primary
          ///< device (On = false), it should do so in this function.
+         ///< A derived class must call the MakePrimaryDevice() function of its
+         ///< base class.
 public:
   bool IsPrimaryDevice(void) const { return this == primaryDevice; }
   int CardIndex(void) const { return cardIndex; }
@@ -183,8 +198,18 @@ public:
   virtual bool HasDecoder(void) const;
          ///< Tells whether this device has an MPEG decoder.
 
+// Device hooks
+
+private:
+  static cList<cDeviceHook> deviceHooks;
+protected:
+  bool DeviceHooksProvidesTransponder(const cChannel *Channel) const;
+
 // SPU facilities
 
+private:
+  cLiveSubtitle *liveSubtitle;
+  cDvbSubtitleConverter *dvbSubtitleConverter;
 public:
   virtual cSpuDecoder *GetSpuDecoder(void);
          ///< Returns a pointer to the device's SPU decoder (or NULL, if this
@@ -219,6 +244,17 @@ public:
          ///< function itself actually returns true.
          ///< The default implementation always returns false, so a derived cDevice
          ///< class that can provide channels must implement this function.
+  virtual int NumProvidedSystems(void) const;
+         ///< Returns the number of individual "delivery systems" this device provides.
+         ///< The default implementation returns 0, so any derived class that can
+         ///< actually provide channels must implement this function.
+         ///< The result of this function is used when selecting a device, in order
+         ///< to avoid devices that provide more than one system.
+  virtual const cChannel *GetCurrentlyTunedTransponder(void) const;
+         ///< Returns a pointer to the currently tuned transponder.
+         ///< This is not one of the channels in the global cChannels list, but rather
+         ///< a local copy. The result may be NULL if the device is not tuned to any
+         ///< transponder.
   virtual bool IsTunedToTransponder(const cChannel *Channel);
          ///< Returns true if this device is currently tuned to the given Channel's
          ///< transponder.
@@ -233,12 +269,12 @@ public:
          ///< Direction (only the sign of Direction is evaluated, positive values
          ///< switch to higher channel numbers).
 private:
+  eSetChannelResult SetChannel(const cChannel *Channel, bool LiveView);
+         ///< Sets the device to the given channel (general setup).
 protected:
   virtual bool SetChannelDevice(const cChannel *Channel, bool LiveView);
          ///< Sets the device to the given channel (actual physical setup).
 public:
-  eSetChannelResult SetChannel(const cChannel *Channel, bool LiveView);
-         ///< Sets the device to the given channel (general setup).
   static int CurrentChannel(void) { return primaryDevice ? currentChannel : 0; }
          ///< Returns the number of the current channel on the primary device.
   static void SetCurrentChannel(const cChannel *Channel) { currentChannel = Channel ? Channel->Number() : 0; }
@@ -266,14 +302,15 @@ protected:
   class cPidHandle {
   public:
     int pid;
+    int streamType;
     int handle;
     int used;
-    cPidHandle(void) { pid = used = 0; handle = -1; }
+    cPidHandle(void) { pid = streamType = used = 0; handle = -1; }
     };
   cPidHandle pidHandles[MAXPIDHANDLES];
   bool HasPid(int Pid) const;
          ///< Returns true if this device is currently receiving the given PID.
-  bool AddPid(int Pid, ePidType PidType = ptOther);
+  bool AddPid(int Pid, ePidType PidType = ptOther, int StreamType = 0);
          ///< Adds a PID to the set of PIDs this device shall receive.
   void DelPid(int Pid, ePidType PidType = ptOther);
          ///< Deletes a PID from the set of PIDs this device shall receive.
@@ -297,12 +334,22 @@ private:
 protected:
   void StartSectionHandler(void);
        ///< A derived device that provides section data must call
-       ///< this function to actually set up the section handler.
+       ///< this function (typically in its constructor) to actually set
+       ///< up the section handler.
+  void StopSectionHandler(void);
+       ///< A device that has called StartSectionHandler() must call this
+       ///< function (typically in its destructor) to stop the section
+       ///< handler.
 public:
   virtual int OpenFilter(u_short Pid, u_char Tid, u_char Mask);
        ///< Opens a file handle for the given filter data.
        ///< A derived device that provides section data must
        ///< implement this function.
+  virtual void CloseFilter(int Handle);
+       ///< Closes a file handle that has previously been opened
+       ///< by OpenFilter(). If this is as simple as calling close(Handle),
+       ///< a derived class need not implement this function, because this
+       ///< is done by the default implementation.
   void AttachFilter(cFilter *Filter);
        ///< Attaches the given filter to this device.
   void Detach(cFilter *Filter);
@@ -356,18 +403,42 @@ public:
   virtual eVideoSystem GetVideoSystem(void);
          ///< Returns the video system of the currently displayed material
          ///< (default is PAL).
+  virtual void GetVideoSize(int &Width, int &Height, double &VideoAspect);
+         ///< Returns the With, Height and VideoAspect ratio of the currently
+         ///< displayed video material. The data returned by this function is
+         ///< only used for informational purposes (if any). Width and
+         ///< Height are given in pixel (e.g. 720x576) and VideoAspect is
+         ///< e.g. 1.33333 for a 4:3 broadcast, or 1.77778 for 16:9.
+         ///< The default implementation returns 0 for Width and Height
+         ///< and 1.0 for VideoAspect.
+  virtual void GetOsdSize(int &Width, int &Height, double &PixelAspect);
+         ///< Returns the With, Height and PixelAspect ratio the OSD should use
+         ///< to best fit the resolution of the output device. If PixelAspect
+         ///< is not 1.0, the OSD may take this as a hint to scale its
+         ///< graphics in a way that, e.g., a circle will actually
+         ///< show up as a circle on the screen, and not as an ellipse.
+         ///< Values greater than 1.0 mean to stretch the graphics in the
+         ///< vertical direction (or shrink it in the horizontal direction,
+         ///< depending on which dimension shall be fixed). Values less than
+         ///< 1.0 work the other way round. Note that the OSD is not guaranteed
+         ///< to actually use this hint.
 
 // Track facilities
 
 private:
   tTrackId availableTracks[ttMaxTrackTypes];
   eTrackType currentAudioTrack;
+  eTrackType currentSubtitleTrack;
   cMutex mutexCurrentAudioTrack;
+  cMutex mutexCurrentSubtitleTrack;
   int currentAudioTrackMissingCount;
-  bool pre_1_3_19_PrivateStream;
+  bool autoSelectPreferredSubtitleLanguage;
+  int pre_1_3_19_PrivateStream;
 protected:
   virtual void SetAudioTrackDevice(eTrackType Type);
        ///< Sets the current audio track to the given value.
+  virtual void SetSubtitleTrackDevice(eTrackType Type);
+       ///< Sets the current subtitle track to the given value.
 public:
   void ClrAvailableTracks(bool DescriptionsOnly = false, bool IdsOnly = false);
        ///< Clears the list of currently availabe tracks. If DescriptionsOnly
@@ -384,18 +455,33 @@ public:
   const tTrackId *GetTrack(eTrackType Type);
        ///< Returns a pointer to the given track id, or NULL if Type is not
        ///< less than ttMaxTrackTypes.
+  int NumTracks(eTrackType FirstTrack, eTrackType LastTrack) const;
+       ///< Returns the number of tracks in the given range that are currently
+       ///< available.
   int NumAudioTracks(void) const;
        ///< Returns the number of audio tracks that are currently available.
        ///< This is just for information, to quickly find out whether there
        ///< is more than one audio track.
+  int NumSubtitleTracks(void) const;
+       ///< Returns the number of subtitle tracks that are currently available.
   eTrackType GetCurrentAudioTrack(void) { return currentAudioTrack; }
   bool SetCurrentAudioTrack(eTrackType Type);
        ///< Sets the current audio track to the given Type.
        ///< \return Returns true if Type is a valid audio track, false otherwise.
+  eTrackType GetCurrentSubtitleTrack(void) { return currentSubtitleTrack; }
+  bool SetCurrentSubtitleTrack(eTrackType Type, bool Manual = false);
+       ///< Sets the current subtitle track to the given Type.
+       ///< IF Manual is true, no automatic preferred subtitle language selection
+       ///< will be done for the rest of the current replay session, or until
+       ///< the channel is changed.
+       ///< \return Returns true if Type is a valid subtitle track, false otherwise.
   void EnsureAudioTrack(bool Force = false);
        ///< Makes sure an audio track is selected that is actually available.
        ///< If Force is true, the language and Dolby Digital settings will
        ///< be verified even if the current audio track is available.
+  void EnsureSubtitleTrack(void);
+       ///< Makes sure one of the preferred language subtitle tracks is selected.
+       ///< Only has an effect if Setup.DisplaySubtitles is on.
 
 // Audio facilities
 
@@ -432,8 +518,16 @@ public:
 
 private:
   cPlayer *player;
-  cPesAssembler *pesAssembler;
+  cPatPmtParser patPmtParser;
+  cTsToPes tsToPesVideo;
+  cTsToPes tsToPesAudio;
+  cTsToPes tsToPesSubtitle;
+  cTsToPes tsToPesTeletext;
+  bool isPlayingVideo;
 protected:
+  const cPatPmtParser *PatPmtParser(void) const { return &patPmtParser; }
+       ///< Returns a pointer to the patPmtParser, so that a derived device
+       ///< can use the stream information from it.
   virtual bool CanReplay(void) const;
        ///< Returns true if this device can currently start a replay session.
   virtual bool SetPlayMode(ePlayMode PlayMode);
@@ -443,7 +537,7 @@ protected:
        ///< Plays the given data block as video.
        ///< Data points to exactly one complete PES packet of the given Length.
        ///< PlayVideo() shall process the packet either as a whole (returning
-       ///< Length) or not at all (returning 0 or -1 and setting 'errno' to EAGAIN).
+       ///< Length) or not at all (returning 0 or -1 and setting 'errno' accordingly).
        ///< \return Returns the number of bytes actually taken from Data, or -1
        ///< in case of an error.
   virtual int PlayAudio(const uchar *Data, int Length, uchar Id);
@@ -451,7 +545,14 @@ protected:
        ///< Data points to exactly one complete PES packet of the given Length.
        ///< Id indicates the type of audio data this packet holds.
        ///< PlayAudio() shall process the packet either as a whole (returning
-       ///< Length) or not at all (returning 0 or -1 and setting 'errno' to EAGAIN).
+       ///< Length) or not at all (returning 0 or -1 and setting 'errno' accordingly).
+       ///< \return Returns the number of bytes actually taken from Data, or -1
+       ///< in case of an error.
+  virtual int PlaySubtitle(const uchar *Data, int Length);
+       ///< Plays the given data block as a subtitle.
+       ///< Data points to exactly one complete PES packet of the given Length.
+       ///< PlaySubtitle() shall process the packet either as a whole (returning
+       ///< Length) or not at all (returning 0 or -1 and setting 'errno' accordingly).
        ///< \return Returns the number of bytes actually taken from Data, or -1
        ///< in case of an error.
   virtual int PlayPesPacket(const uchar *Data, int Length, bool VideoOnly = false);
@@ -459,11 +560,48 @@ protected:
        ///< If VideoOnly is true, only the video will be displayed,
        ///< which is necessary for trick modes like 'fast forward'.
        ///< Data must point to one single, complete PES packet.
+  virtual int PlayTsVideo(const uchar *Data, int Length);
+       ///< Plays the given data block as video.
+       ///< Data points to exactly one complete TS packet of the given Length
+       ///< (which is always TS_SIZE).
+       ///< PlayTsVideo() shall process the packet either as a whole (returning
+       ///< Length) or not at all (returning 0 or -1 and setting 'errno' accordingly).
+       ///< The default implementation collects all incoming TS payload belonging
+       ///< to one PES packet and calls PlayVideo() with the resulting packet.
+  virtual int PlayTsAudio(const uchar *Data, int Length);
+       ///< Plays the given data block as audio.
+       ///< Data points to exactly one complete TS packet of the given Length
+       ///< (which is always TS_SIZE).
+       ///< PlayTsAudio() shall process the packet either as a whole (returning
+       ///< Length) or not at all (returning 0 or -1 and setting 'errno' accordingly).
+       ///< The default implementation collects all incoming TS payload belonging
+       ///< to one PES packet and calls PlayAudio() with the resulting packet.
+  virtual int PlayTsSubtitle(const uchar *Data, int Length);
+       ///< Plays the given data block as a subtitle.
+       ///< Data points to exactly one complete TS packet of the given Length
+       ///< (which is always TS_SIZE).
+       ///< PlayTsSubtitle() shall process the packet either as a whole (returning
+       ///< Length) or not at all (returning 0 or -1 and setting 'errno' accordingly).
+       ///< The default implementation collects all incoming TS payload belonging
+       ///< to one PES packet and displays the resulting subtitle via the OSD.
 public:
   virtual int64_t GetSTC(void);
        ///< Gets the current System Time Counter, which can be used to
        ///< synchronize audio and video. If this device is unable to
        ///< provide the STC, -1 will be returned.
+       ///< The value returned doesn't need to be an actual "clock" value,
+       ///< it is sufficient if it holds the PTS (Presentation Time Stamp) of
+       ///< the most recently presented frame. A proper value must be returned
+       ///< in normal replay mode as well as in any trick modes (like slow motion,
+       ///< fast forward/rewind).
+       ///< Only the lower 32 bit of this value are actually used, since some
+       ///< devices can't handle the msb correctly.
+  virtual bool IsPlayingVideo(void) const { return isPlayingVideo; }
+       ///< \return Returns true if the currently attached player has delivered
+       ///< any video packets.
+  virtual bool HasIBPTrickSpeed(void) { return false; }
+       ///< Returns true if this device can handle all frames in 'fast forward'
+       ///< trick speeds.
   virtual void TrickSpeed(int Speed);
        ///< Sets the device into a mode where replay is done slower.
        ///< Every single frame shall then be displayed the given number of
@@ -489,6 +627,10 @@ public:
        ///< all registered cAudio objects are notified.
   virtual void StillPicture(const uchar *Data, int Length);
        ///< Displays the given I-frame as a still picture.
+       ///< Data points either to TS (first byte is 0x47) or PES (first byte
+       ///< is 0x00) data of the given Length. The default implementation
+       ///< converts TS to PES and calls itself again, allowing a derived class
+       ///< to display PES if it can't handle TS directly.
   virtual bool Poll(cPoller &Poller, int TimeoutMs = 0);
        ///< Returns true if the device itself or any of the file handles in
        ///< Poller is ready for further action.
@@ -499,7 +641,7 @@ public:
        ///< data which was bufferd so far has been processed.
        ///< If TimeoutMs is not zero, the device will wait up to the given
        ///< number of milliseconds before returning in case there is still
-       ///< data in the buffers..
+       ///< data in the buffers.
   virtual int PlayPes(const uchar *Data, int Length, bool VideoOnly = false);
        ///< Plays all valid PES packets in Data with the given Length.
        ///< If Data is NULL any leftover data from a previous call will be
@@ -510,6 +652,22 @@ public:
        ///< to a complete packet with data from the next call to PlayPes().
        ///< That way any functions called from within PlayPes() will be
        ///< guaranteed to always receive complete PES packets.
+  virtual int PlayTs(const uchar *Data, int Length, bool VideoOnly = false);
+       ///< Plays the given TS packet.
+       ///< If VideoOnly is true, only the video will be displayed,
+       ///< which is necessary for trick modes like 'fast forward'.
+       ///< Data points to a single TS packet, Length is always TS_SIZE (the total
+       ///< size of a single TS packet).
+       ///< If Data is NULL any leftover data from a previous call will be
+       ///< discarded.
+       ///< A derived device can reimplement this function to handle the
+       ///< TS packets itself. Any packets the derived function can't handle
+       ///< must be sent to the base class function. This applies especially
+       ///< to the PAT/PMT packets.
+       ///< Returns -1 in case of error, otherwise the number of actually
+       ///< processed bytes is returned.
+       ///< PlayTs() shall process the TS packets either as a whole (returning
+       ///< n*TS_SIZE) or not at all, returning 0 or -1 and setting 'errno' accordingly).
   bool Replaying(void) const;
        ///< Returns true if we are currently replaying.
   bool Transferring(void) const;
