@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: osd.c 1.71 2007/07/20 14:51:36 kls Exp $
+ * $Id: osd.c 2.10 2010/05/02 13:56:53 kls Exp $
  */
 
 #include "osd.h"
@@ -14,9 +14,8 @@
 #include <sys/ioctl.h>
 #include <sys/stat.h>
 #include <sys/unistd.h>
+#include "device.h"
 #include "tools.h"
-#include "osdcontroller.h"
-#include "vdrttxtsubshooks.h"
 
 // --- cPalette --------------------------------------------------------------
 
@@ -26,6 +25,10 @@ cPalette::cPalette(int Bpp)
   SetAntiAliasGranularity(10, 10);
 }
 
+cPalette::~cPalette()
+{
+}
+
 void cPalette::SetAntiAliasGranularity(uint FixedColors, uint BlendColors)
 {
   if (FixedColors >= MAXNUMCOLORS || BlendColors == 0)
@@ -84,7 +87,7 @@ void cPalette::SetColor(int Index, tColor Color)
      }
 }
 
-const tColor *cPalette::Colors(int &NumColors)
+const tColor *cPalette::Colors(int &NumColors) const
 {
   NumColors = numColors;
   return numColors ? color : NULL;
@@ -98,6 +101,7 @@ void cPalette::Take(const cPalette &Palette, tIndexes *Indexes, tColor ColorFg,
          switch (i) {
            case 0: Color = ColorBg; break;
            case 1: Color = ColorFg; break;
+           default: ;
            }
          }
       int n = Index(Color);
@@ -114,7 +118,7 @@ void cPalette::Replace(const cPalette &Palette)
   antiAliasGranularity = Palette.antiAliasGranularity;
 }
 
-tColor cPalette::Blend(tColor ColorFg, tColor ColorBg, uint8_t Level)
+tColor cPalette::Blend(tColor ColorFg, tColor ColorBg, uint8_t Level) const
 {
   if (antiAliasGranularity > 0)
      Level = uint8_t(int(Level / antiAliasGranularity + 0.5) * antiAliasGranularity);
@@ -133,7 +137,7 @@ tColor cPalette::Blend(tColor ColorFg, tColor ColorBg, uint8_t Level)
   return (A << 24) | (R << 16) | (G << 8) | B;
 }
 
-int cPalette::ClosestColor(tColor Color, int MaxDiff)
+int cPalette::ClosestColor(tColor Color, int MaxDiff) const
 {
   int n = 0;
   int d = INT_MAX;
@@ -141,12 +145,14 @@ int cPalette::ClosestColor(tColor Color, int MaxDiff)
   int R1 = (Color & 0x00FF0000) >> 16;
   int G1 = (Color & 0x0000FF00) >>  8;
   int B1 = (Color & 0x000000FF);
-  for (int i = 0; i < numColors; i++) {
+  for (int i = 0; i < numColors && d > 0; i++) {
       int A2 = (color[i] & 0xFF000000) >> 24;
       int R2 = (color[i] & 0x00FF0000) >> 16;
       int G2 = (color[i] & 0x0000FF00) >>  8;
       int B2 = (color[i] & 0x000000FF);
-      int diff = (abs(A1 - A2) << 1) + (abs(R1 - R2) << 1) + (abs(G1 - G2) << 1) + (abs(B1 - B2) << 1);
+      int diff = 0;
+      if (A1 || A2) // fully transparent colors are considered equal
+         diff = (abs(A1 - A2) << 1) + (abs(R1 - R2) << 1) + (abs(G1 - G2) << 1) + (abs(B1 - B2) << 1);
       if (diff < d) {
          d = diff;
          n = i;
@@ -323,6 +329,8 @@ bool cBitmap::LoadXpm(const char *FileName)
 
 bool cBitmap::SetXpm(const char *const Xpm[], bool IgnoreNone)
 {
+  if (!Xpm)
+     return false;
   const char *const *p = Xpm;
   int w, h, n, c;
   if (4 != sscanf(*p, "%d %d %d %d", &w, &h, &n, &c)) {
@@ -352,10 +360,10 @@ bool cBitmap::SetXpm(const char *const Xpm[], bool IgnoreNone)
          }
       s = skipspace(s + 1);
       if (strcasecmp(s, "none") == 0) {
-         s = "#00000000";
          NoneColorIndex = i;
-         if (IgnoreNone)
-            continue;
+         if (!IgnoreNone)
+            SetColor(i, clrTransparent);
+         continue;
          }
       if (*s != '#') {
          esyslog("ERROR: unknown color code in XPM: '%c'", *s);
@@ -449,13 +457,13 @@ void cBitmap::DrawText(int x, int y, const char *s, tColor ColorFg, tColor Color
      int w = Font->Width(s);
      int h = Font->Height();
      int limit = 0;
+     int cw = Width ? Width : w;
+     int ch = Height ? Height : h;
+     if (!Intersects(x, y, x + cw - 1, y + ch - 1))
+        return;
+     if (ColorBg != clrTransparent)
+        DrawRectangle(x, y, x + cw - 1, y + ch - 1, ColorBg);
      if (Width || Height) {
-        int cw = Width ? Width : w;
-        int ch = Height ? Height : h;
-        if (!Intersects(x, y, x + cw - 1, y + ch - 1))
-           return;
-        if (ColorBg != clrTransparent)
-           DrawRectangle(x, y, x + cw - 1, y + ch - 1, ColorBg);
         limit = x + cw - x0;
         if (Width) {
            if ((Alignment & taLeft) != 0)
@@ -482,8 +490,6 @@ void cBitmap::DrawText(int x, int y, const char *s, tColor ColorFg, tColor Color
               }
            }
         }
-     else if (!Intersects(x, y, x + w - 1, y + h - 1))
-        return;
      x -= x0;
      y -= y0;
      Font->DrawText(this, x, y, s, ColorFg, ColorBg, limit);
@@ -529,6 +535,7 @@ void cBitmap::DrawEllipse(int x1, int y1, int x2, int y2, tColor Color, int Quad
     case 6:          cy = y2; rx /= 2; break;
     case 7: cx = x2;          ry /= 2; break;
     case 8:          cy = y1; rx /= 2; break;
+    default: ;
     }
   int TwoASquare = 2 * rx * rx;
   int TwoBSquare = 2 * ry * ry;
@@ -554,6 +561,7 @@ void cBitmap::DrawEllipse(int x1, int y1, int x2, int y2, tColor Color, int Quad
           case -2: DrawRectangle(x1,     cy - y, cx - x, cy - y, Color); break;
           case -3: DrawRectangle(x1,     cy + y, cx - x, cy + y, Color); break;
           case -4: DrawRectangle(cx + x, cy + y, x2,     cy + y, Color); break;
+          default: ;
           }
         y++;
         StoppingY += TwoASquare;
@@ -588,6 +596,7 @@ void cBitmap::DrawEllipse(int x1, int y1, int x2, int y2, tColor Color, int Quad
           case -2: DrawRectangle(x1,     cy - y, cx - x, cy - y, Color); break;
           case -3: DrawRectangle(x1,     cy + y, cx - x, cy + y, Color); break;
           case -4: DrawRectangle(cx + x, cy + y, x2,     cy + y, Color); break;
+          default: ;
           }
         x++;
         StoppingX += TwoBSquare;
@@ -637,31 +646,120 @@ void cBitmap::DrawSlope(int x1, int y1, int x2, int y2, tColor Color, int Type)
      }
 }
 
-const tIndex *cBitmap::Data(int x, int y)
+const tIndex *cBitmap::Data(int x, int y) const
 {
   return &bitmap[y * width + x];
 }
 
+void cBitmap::ReduceBpp(const cPalette &Palette)
+{
+  int NewBpp = Palette.Bpp();
+  if (Bpp() == 4 && NewBpp == 2) {
+     for (int i = width * height; i--; ) {
+         tIndex p = bitmap[i];
+         bitmap[i] = (p >> 2) | ((p & 0x03) != 0);
+         }
+     }
+  else if (Bpp() == 8) {
+     if (NewBpp == 2) {
+        for (int i = width * height; i--; ) {
+            tIndex p = bitmap[i];
+            bitmap[i] = (p >> 6) | ((p & 0x30) != 0);
+            }
+        }
+     else if (NewBpp == 4) {
+        for (int i = width * height; i--; ) {
+            tIndex p = bitmap[i];
+            bitmap[i] = p >> 4;
+            }
+        }
+     else
+        return;
+     }
+  else
+     return;
+  SetBpp(NewBpp);
+  Replace(Palette);
+}
+
+void cBitmap::ShrinkBpp(int NewBpp)
+{
+  int NumOldColors;
+  const tColor *Colors = this->Colors(NumOldColors);
+  if (Colors) {
+     // Find the most frequently used colors and create a map table:
+     int Used[MAXNUMCOLORS] = { 0 };
+     int Map[MAXNUMCOLORS] = { 0 };
+     for (int i = width * height; i--; )
+         Used[bitmap[i]]++;
+     int MaxNewColors = (NewBpp == 4) ? 16 : 4;
+     cPalette NewPalette(NewBpp);
+     for (int i = 0; i < MaxNewColors; i++) {
+         int Max = 0;
+         int Index = -1;
+         for (int n = 0; n < NumOldColors; n++) {
+             if (Used[n] > Max) {
+                Max = Used[n];
+                Index = n;
+                }
+             }
+         if (Index >= 0) {
+            Used[Index] = 0;
+            Map[Index] = i;
+            NewPalette.SetColor(i, Colors[Index]);
+            }
+         else
+            break;
+         }
+     // Complete the map table for all other colors (will be set to closest match):
+     for (int n = 0; n < NumOldColors; n++) {
+         if (Used[n])
+            Map[n] = NewPalette.Index(Colors[n]);
+         }
+     // Do the actual index mapping:
+     for (int i = width * height; i--; )
+         bitmap[i] = Map[bitmap[i]];
+     SetBpp(NewBpp);
+     Replace(NewPalette);
+     }
+}
+
 // --- cOsd ------------------------------------------------------------------
 
+static const char *OsdErrorTexts[] = {
+  "ok",
+  "too many areas",
+  "too many colors",
+  "bpp not supported",
+  "areas overlap",
+  "wrong alignment",
+  "out of memory",
+  "wrong area size",
+  "unknown",
+  };
+
 int cOsd::osdLeft = 0;
 int cOsd::osdTop = 0;
 int cOsd::osdWidth = 0;
 int cOsd::osdHeight = 0;
-int cOsd::isOpen = 0;
-bool cOsd::niosd  = false;
+cVector<cOsd *> cOsd::Osds;
 
-cOsd::cOsd(int Left, int Top)
+cOsd::cOsd(int Left, int Top, uint Level)
 {
-  if (isOpen)
-     esyslog("ERROR: OSD opened without closing previous OSD!");
   savedRegion = NULL;
   numBitmaps = 0;
   left = Left;
   top = Top;
   width = height = 0;
-  isOpen++;
-  vidWin.bpp = 0;
+  level = Level;
+  active = false;
+  for (int i = 0; i < Osds.Size(); i++) {
+      if (Osds[i]->level > level) {
+         Osds.Insert(this, i);
+         return;
+         }
+      }
+  Osds.Append(this);
 }
 
 cOsd::~cOsd()
@@ -669,10 +767,14 @@ cOsd::~cOsd()
   for (int i = 0; i < numBitmaps; i++)
       delete bitmaps[i];
   delete savedRegion;
-  isOpen--;
-  if (!niosd)
-     NonInteractiveOsdPatch::OsdController.Show();
-  cVDRTtxtsubsHookListener::Hook()->ShowOSD();
+  for (int i = 0; i < Osds.Size(); i++) {
+      if (Osds[i] == this) {
+         Osds.Remove(i);
+         if (Osds.Size())
+            Osds[0]->SetActive(true);
+         break;
+         }
+      }
 }
 
 void cOsd::SetOsdPosition(int Left, int Top, int Width, int Height)
@@ -714,20 +816,19 @@ eOsdError cOsd::CanHandleAreas(const tArea *Areas, int NumAreas)
 
 eOsdError cOsd::SetAreas(const tArea *Areas, int NumAreas)
 {
-  eOsdError Result = oeUnknown;
-  if (numBitmaps == 0) {
-     Result = CanHandleAreas(Areas, NumAreas);
-     if (Result == oeOk) {
-        width = height = 0;
-        for (int i = 0; i < NumAreas; i++) {
-            bitmaps[numBitmaps++] = new cBitmap(Areas[i].Width(), Areas[i].Height(), Areas[i].bpp, Areas[i].x1, Areas[i].y1);
-            width = max(width, Areas[i].x2 + 1);
-            height = max(height, Areas[i].y2 + 1);
-            }
-        }
+  eOsdError Result = CanHandleAreas(Areas, NumAreas);
+  if (Result == oeOk) {
+     while (numBitmaps)
+           delete bitmaps[--numBitmaps];
+     width = height = 0;
+     for (int i = 0; i < NumAreas; i++) {
+         bitmaps[numBitmaps++] = new cBitmap(Areas[i].Width(), Areas[i].Height(), Areas[i].bpp, Areas[i].x1, Areas[i].y1);
+         width = max(width, Areas[i].x2 + 1);
+         height = max(height, Areas[i].y2 + 1);
+         }
      }
-  if (Result != oeOk)
-     esyslog("ERROR: cOsd::SetAreas returned %d", Result);
+  else
+     esyslog("ERROR: cOsd::SetAreas returned %d (%s)", Result, Result < oeUnknown ? OsdErrorTexts[Result] : OsdErrorTexts[oeUnknown]);
   return Result;
 }
 
@@ -798,6 +899,9 @@ void cOsd::Flush(void)
 // --- cOsdProvider ----------------------------------------------------------
 
 cOsdProvider *cOsdProvider::osdProvider = NULL;
+int cOsdProvider::oldWidth = 0;
+int cOsdProvider::oldHeight = 0;
+double cOsdProvider::oldAspect = 1.0;
 
 cOsdProvider::cOsdProvider(void)
 {
@@ -810,20 +914,48 @@ cOsdProvider::~cOsdProvider()
   osdProvider = NULL;
 }
 
-cOsd *cOsdProvider::NewOsd(int Left, int Top, bool dontHide)
+cOsd *cOsdProvider::NewOsd(int Left, int Top, uint Level)
 {
-  if (!dontHide)
-     NonInteractiveOsdPatch::OsdController.Hide();
-  cOsd::niosd = dontHide;
-  // OSD_HOOK_2 - Information to Checkpatch.sh
-  cVDRTtxtsubsHookListener::Hook()->HideOSD();
-  if (cOsd::IsOpen())
+  if (Level == OSD_LEVEL_DEFAULT && cOsd::IsOpen())
      esyslog("ERROR: attempt to open OSD while it is already open - using dummy OSD!");
-  else if (osdProvider)
-     return osdProvider->CreateOsd(Left, Top);
+  else if (osdProvider) {
+     cOsd *ActiveOsd = cOsd::Osds.Size() ? cOsd::Osds[0] : NULL;
+     cOsd *Osd = osdProvider->CreateOsd(Left, Top, Level);
+     if (Osd == cOsd::Osds[0]) {
+        if (ActiveOsd)
+           ActiveOsd->SetActive(false);
+        Osd->SetActive(true);
+        }
+     return Osd;
+     }
   else
      esyslog("ERROR: no OSD provider available - using dummy OSD!");
-  return new cOsd(Left, Top); // create a dummy cOsd, so that access won't result in a segfault
+  return new cOsd(Left, Top, 999); // create a dummy cOsd, so that access won't result in a segfault
+}
+
+void cOsdProvider::UpdateOsdSize(bool Force)
+{
+  int Width;
+  int Height;
+  double Aspect;
+  cDevice::PrimaryDevice()->GetOsdSize(Width, Height, Aspect);
+  if (Width != oldWidth || Height != oldHeight || !DoubleEqual(Aspect, oldAspect) || Force) {
+     Setup.OSDLeft = int(round(Width * Setup.OSDLeftP));
+     Setup.OSDTop = int(round(Height * Setup.OSDTopP));
+     Setup.OSDWidth = int(round(Width * Setup.OSDWidthP)) & ~0x07; // OSD width must be a multiple of 8
+     Setup.OSDHeight = int(round(Height * Setup.OSDHeightP));
+     Setup.OSDAspect = Aspect;
+     Setup.FontOsdSize = int(round(Height * Setup.FontOsdSizeP));
+     Setup.FontFixSize = int(round(Height * Setup.FontFixSizeP));
+     Setup.FontSmlSize = int(round(Height * Setup.FontSmlSizeP));
+     cFont::SetFont(fontOsd, Setup.FontOsd, Setup.FontOsdSize);
+     cFont::SetFont(fontFix, Setup.FontFix, Setup.FontFixSize);
+     cFont::SetFont(fontSml, Setup.FontSml, Setup.FontSmlSize);
+     oldWidth = Width;
+     oldHeight = Height;
+     oldAspect = Aspect;
+     dsyslog("OSD size changed to %dx%d @ %g", Width, Height, Aspect);
+     }
 }
 
 void cOsdProvider::Shutdown(void)
