@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: osd.h 1.56 2007/07/20 14:50:17 kls Exp $
+ * $Id: osd.h 2.5 2010/01/17 13:23:50 kls Exp $
  */
 
 #ifndef __OSD_H
@@ -15,6 +15,10 @@
 #include <stdint.h>
 #include "config.h"
 #include "font.h"
+#include "tools.h"
+
+#define OSD_LEVEL_DEFAULT     0
+#define OSD_LEVEL_SUBTITLES  10
 
 #define MAXNUMCOLORS 256
 
@@ -32,7 +36,7 @@ enum {
   clrWhite       = 0xFFFCFCFC,
   };
 
-enum eOsdError { oeOk,
+enum eOsdError { oeOk, // see also OsdErrorTexts in osd.c
                  oeTooManyAreas,
                  oeTooManyColors,
                  oeBppNotSupported,
@@ -58,6 +62,7 @@ protected:
 public:
   cPalette(int Bpp = 8);
         ///< Initializes the palette with the given color depth.
+  virtual ~cPalette();
   void SetAntiAliasGranularity(uint FixedColors, uint BlendColors);
         ///< Allows the system to optimize utilization of the limited color
         ///< palette entries when generating blended colors for anti-aliasing.
@@ -68,7 +73,7 @@ public:
         ///< a single color combination, and may not be able to serve all
         ///< requested colors. By default the palette assumes there will be
         ///< 10 fixed colors and 10 color combinations.
-  int Bpp(void) { return bpp; }
+  int Bpp(void) const { return bpp; }
   void Reset(void);
         ///< Resets the palette, making it contain no colors.
   int Index(tColor Color);
@@ -76,7 +81,7 @@ public:
         ///< If Color is not yet contained in this palette, it will be added if
         ///< there is a free slot. If the color can't be added to this palette,
         ///< the closest existing color will be returned.
-  tColor Color(int Index) { return Index < maxColors ? color[Index] : 0; }
+  tColor Color(int Index) const { return Index < maxColors ? color[Index] : 0; }
         ///< Returns the color at the given Index. If Index is outside the valid
         ///< range, 0 will be returned.
   void SetBpp(int Bpp);
@@ -86,7 +91,7 @@ public:
         ///< Sets the palette entry at Index to Color. If Index is larger than
         ///< the number of currently used entries in this palette, the entries
         ///< in between will have undefined values.
-  const tColor *Colors(int &NumColors);
+  const tColor *Colors(int &NumColors) const;
         ///< Returns a pointer to the complete color table and stores the
         ///< number of valid entries in NumColors. If no colors have been
         ///< stored yet, NumColors will be set to 0 and the function will
@@ -101,14 +106,14 @@ public:
   void Replace(const cPalette &Palette);
         ///< Replaces the colors of this palette with the colors from the given
         ///< palette.
-  tColor Blend(tColor ColorFg, tColor ColorBg, uint8_t Level);
+  tColor Blend(tColor ColorFg, tColor ColorBg, uint8_t Level) const;
         ///< Determines a color that consists of a linear blend between ColorFg
         ///< and ColorBg. If Level is 0, the result is ColorBg, if it is 255,
         ///< the result is ColorFg. If SetAntiAliasGranularity() has been called previously,
         ///< Level will be mapped to a limited range of levels that allow to make best
         ///< use of the palette entries.
-  int ClosestColor(tColor Color, int MaxDiff = INT_MAX);
-        ///< Returns the index of a color in this paltte that is closest to the given
+  int ClosestColor(tColor Color, int MaxDiff = INT_MAX) const;
+        ///< Returns the index of a color in this palette that is closest to the given
         ///< Color. MaxDiff can be used to control the maximum allowed color difference.
         ///< If no color with a maximum difference of MaxDiff can be found, -1 will
         ///< be returned. With the default value of INT_MAX, there will always be
@@ -227,10 +232,21 @@ public:
        ///< 5: vertical,   rising,  upper
        ///< 6: vertical,   falling, lower
        ///< 7: vertical,   falling, upper
-  const tIndex *Data(int x, int y);
+  const tIndex *Data(int x, int y) const;
        ///< Returns the address of the index byte at the given coordinates.
-  tColor GetColor(int x, int y) { return Color(*Data(x, y)); }
+  tColor GetColor(int x, int y) const { return Color(*Data(x, y)); }
        ///< Returns the color at the given coordinates.
+  void ReduceBpp(const cPalette &Palette);
+       ///< Reduces the color depth of the bitmap to that of the given Palette.
+       ///< If Palette's color depth is not smaller than the bitmap's current
+       ///< color depth, or if it is not one of 4bpp or 2bpp, nothing happens. After
+       ///< reducing the color depth the current palette is replaced with
+       ///< the given one.
+  void ShrinkBpp(int NewBpp);
+       ///< Shrinks the color depth of the bitmap to NewBpp by keeping only
+       ///< the 2^NewBpp most frequently used colors as defined in the current palette.
+       ///< If NewBpp is not smaller than the bitmap's current color depth,
+       ///< or if it is not one of 4bpp or 2bpp, nothing happens.
   };
 
 struct tArea {
@@ -247,15 +263,15 @@ class cOsd {
   friend class cOsdProvider;
 private:
   static int osdLeft, osdTop, osdWidth, osdHeight;
-  static int isOpen;
+  static cVector<cOsd *> Osds;
   cBitmap *savedRegion;
   cBitmap *bitmaps[MAXOSDAREAS];
   int numBitmaps;
   int left, top, width, height;
-public:
-  static bool niosd;
+  uint level;
+  bool active;
 protected:
-  cOsd(int Left, int Top);
+  cOsd(int Left, int Top, uint Level);
        ///< Initializes the OSD with the given coordinates.
        ///< By default it is assumed that the full area will be able to display
        ///< full 32 bit graphics (ARGB with eight bit for each color and the alpha
@@ -271,6 +287,14 @@ protected:
        ///< and should require only the minimum necessary color depth. This is
        ///< because a derived cOsd class may or may not be able to handle more
        ///< than one area.
+       ///< There can be any number of cOsd objects at the same time, but only
+       ///< one of them will be active at any given time. The active OSD is the
+       ///< one with the lowest value of Level. If there are several cOsd objects
+       ///< with the same Level, the one that was created first will be active.
+  bool Active(void) { return active; }
+  virtual void SetActive(bool On) { active = On; }
+       ///< Sets this OSD to be the active one.
+       ///< A derived class must call cOsd::SetActive(On).
 public:
   virtual ~cOsd();
        ///< Shuts down the OSD.
@@ -283,7 +307,8 @@ public:
        ///< This may be useful for plugins that determine the scaling of the
        ///< video image and need to scale the OSD accordingly to fit on the
        ///< screen.
-  static int IsOpen(void) { return isOpen; }
+  static int IsOpen(void) { return Osds.Size() && Osds[0]->level == OSD_LEVEL_DEFAULT; }
+       ///< Returns true if there is currently a level 0 OSD open.
   int Left(void) { return left; }
   int Top(void) { return top; }
   int Width(void) { return width; }
@@ -317,6 +342,7 @@ public:
        ///< If the OSD has been divided into several sub-areas, all areas that
        ///< are part of the rectangle that surrounds a given drawing operation
        ///< will be drawn into, with the proper offsets.
+       ///< A new call overwrites any previous settings
   virtual void SaveRegion(int x1, int y1, int x2, int y2);
        ///< Saves the region defined by the given coordinates for later restoration
        ///< through RestoreRegion(). Only one saved region can be active at any
@@ -375,26 +401,33 @@ public:
        ///< 7: vertical,   falling, upper
   virtual void Flush(void);
        ///< Actually commits all data to the OSD hardware.
-  tArea vidWin;
   };
 
 class cOsdProvider {
 private:
   static cOsdProvider *osdProvider;
+  static int oldWidth;
+  static int oldHeight;
+  static double oldAspect;
 protected:
-  virtual cOsd *CreateOsd(int Left, int Top) = 0;
+  virtual cOsd *CreateOsd(int Left, int Top, uint Level) = 0;
       ///< Returns a pointer to a newly created cOsd object, which will be located
       ///< at the given coordinates.
 public:
   cOsdProvider(void);
       //XXX maybe parameter to make this one "sticky"??? (frame-buffer etc.)
   virtual ~cOsdProvider();
-  static cOsd *NewOsd(int Left, int Top, bool dontHide = false);
+  static cOsd *NewOsd(int Left, int Top, uint Level = OSD_LEVEL_DEFAULT);
       ///< Returns a pointer to a newly created cOsd object, which will be located
       ///< at the given coordinates. When the cOsd object is no longer needed, the
       ///< caller must delete it. If the OSD is already in use, or there is no OSD
       ///< provider, a dummy OSD is returned so that the caller may always use the
       ///< returned pointer without having to check it every time it is accessed.
+  static void UpdateOsdSize(bool Force = false);
+      ///< Inquires the actual size of the video display and adjusts the OSD and
+      ///< font sizes accordingly. If Force is true, all settings are recalculated,
+      ///< even if the video resolution hasn't changed since the last call to
+      ///< this funtion.
   static void Shutdown(void);
       ///< Shuts down the OSD provider facility by deleting the current OSD provider.
   };
