@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: osdbase.c 1.31 2007/06/09 10:07:46 kls Exp $
+ * $Id: osdbase.c 2.2 2010/01/17 11:36:12 kls Exp $
  */
 
 #include "osdbase.h"
@@ -41,7 +41,7 @@ cOsdItem::~cOsdItem()
 void cOsdItem::SetText(const char *Text, bool Copy)
 {
   free(text);
-  text = Copy ? strdup(Text) : (char *)Text; // text assumes ownership!
+  text = Copy ? strdup(Text ? Text : "") : (char *)Text; // text assumes ownership!
 }
 
 void cOsdItem::SetSelectable(bool Selectable)
@@ -218,6 +218,7 @@ void cOsdMenu::Display(void)
          }
      if (current < 0)
         current = 0; // just for safety - there HAS to be a current item!
+     first = min(first, max(0, count - displayMenuItems)); // in case the menu size has changed
      if (current - first >= displayMenuItems || current < first) {
         first = current - displayMenuItems / 2;
         if (first + displayMenuItems > count)
@@ -237,6 +238,7 @@ void cOsdMenu::Display(void)
          i++;
          }
      }
+  displayMenu->SetScrollbar(count, first);
   if (!isempty(status))
      displayMenu->SetMessage(mtStatus, status);
 }
